;**** Processor Definition
.include "m8def.inc"
.equ		CPUFREQ = 16000								; processor frequency for the delay routines
.equ		INTERRUPTS = 1
;**** pin assignment file
.include "pinconf.inc"
;**** display commands
.include "asciidisp.inc"

;*****************************************************************************************
;	Variablen
;*****************************************************************************************

.def	Null			= r2							; store a zero
.def	One				= r3							; store a one
.def	NegOne			= r4							; store a "-1"
.def	Temp2L			= r14							; temporary register
.def	Temp1L			= r15							; temporary register
.def	Temp			= r16							; temporary register
.def	Temp2			= r17							; temporary register
.def	Temp3			= r18							; temporary register
.def	Temp4			= r19							; temporary register
.def	Temp5			= r20							; temporary register
.def	Temp6			= r21							; temporary register
.def	Temp7			= r22
.def	Status			= r23							; program state
															; bit	0 1=no further input pulses,
															;		  start output calculation and generation
															; bit	1 1=start slow routines (display and key routines)
															; bits23  0=transmit 1st adress byte
															;         1=transmit 2nd adress byte
															;         2=write data
															;         3=read data
															; bit	4 1=value changed, save after menu finished
															; bit	5 0=wait for synchronisation
															;		  1=synchronised, finish if overflow
															; bit	6 0=channel 1 input
															;		  1=channel 2 input

;*****************************************************************************************
;	SRAM-Variablen
;*****************************************************************************************

.equ	Channel1_1H		= 0x60
.equ	Channel1_1L		= 0x61							; Channel 1 of source 1
;.equ	Channel1_2H		= 0x62
;.equ	Channel1_2L		= 0x63							; Channel 2 of source 1
;.equ	Channel1_3H		= 0x64
;.equ	Channel1_3L		= 0x65							; Channel 3 of source 1
;.equ	Channel1_4H		= 0x66
;.equ	Channel1_4L		= 0x67							; Channel 4 of source 1
;.equ	Channel1_5H		= 0x68
;.equ	Channel1_5L		= 0x69							; Channel 5 of source 1
;.equ	Channel1_6H		= 0x6A
;.equ	Channel1_6L		= 0x6B							; Channel 6 of source 1
;.equ	Channel1_7H		= 0x6C
;.equ	Channel1_7L		= 0x6D							; Channel 7 of source 1
;.equ	Channel1_8H		= 0x6E
;.equ	Channel1_8L		= 0x6F							; Channel 8 of source 1
.equ	Channel2_1L		= 0x70							; Channel 1 of source 2
;.equ	Channel2_1H		= 0x71
;.equ	Channel2_2L		= 0x72							; Channel 2 of source 2
;.equ	Channel2_2H		= 0x73
;.equ	Channel2_3L		= 0x74							; Channel 3 of source 2
;.equ	Channel2_3H		= 0x75
;.equ	Channel2_4L		= 0x76							; Channel 4 of source 2
;.equ	Channel2_4H		= 0x77
;.equ	Channel2_5L		= 0x78							; Channel 5 of source 2
;.equ	Channel2_5H		= 0x79
;.equ	Channel2_6L		= 0x7A							; Channel 6 of source 2
;.equ	Channel2_6H		= 0x7B
;.equ	Channel2_7L		= 0x7C							; Channel 7 of source 2
;.equ	Channel2_7H		= 0x7D
.equ	Channel2_8L		= 0x7E							; Channel 8 of source 2
;.equ	Channel2_8H		= 0x7F
.equ	Channelo_1L		= 0x80							; Channel 1 output
.equ	Channelo_1H		= 0x81
;.equ	Channelo_2L		= 0x82							; Channel 2 output
;.equ	Channelo_2H		= 0x83
;.equ	Channelo_3L		= 0x84							; Channel 3 output
;.equ	Channelo_3H		= 0x85
;.equ	Channelo_4L		= 0x86							; Channel 4 output
;.equ	Channelo_4H		= 0x87
;.equ	Channelo_5L		= 0x88							; Channel 5 output
;.equ	Channelo_5H		= 0x89
;.equ	Channelo_6L		= 0x8A							; Channel 6 output
;.equ	Channelo_6H		= 0x8B
;.equ	Channelo_7L		= 0x8C							; Channel 7 output
;.equ	Channelo_7H		= 0x8D
;.equ	Channelo_8L		= 0x8E							; Channel 8 output
;.equ	Channelo_8H		= 0x8F
.equ	Channel1ActPulse= 0x90							; actual channel of source 1
.equ	Channel2ActPulse= 0x91							; actual channel of source 2
.equ	ChanneloActPulse= 0x92							; actual channel of output
.equ	LastCaptureL	= 0x93							; last capture time
.equ	LastCaptureH	= 0x94
.equ	NextSequenceSL	= 0x95							; start time of the next output sequence
.equ	NextSequenceSH	= 0x96
.equ	T1OvfCnt		= 0x97							; timer 1 overflow counter to activate slow routines
.equ	DispDataZeros	= 0x98							; used for display control
.equ	DispDataOnes	= 0x99
.equ	Status3			= 0x9A							; menu state
															; bits 3210
															;   0 display inputs, normal state
															;   1 display outputs, normal state
															;   2 set model
															;   3 init new model
															;   4 set model name
															;   5 set min channel value
															;   6 set max channel value
															;   7 set channel offset
															;   8 set channel multipliers
															;   9 set master channels
															; bit4: 1= main menu active, switch between different modes
															; bit5: 1= choose multipliers submenu active, switch between the target channels
.equ	Status4			= 0x9B							; actual menu item
															; lower 4 bits: target channel
															; higher 4 bits: source channel
.equ	SwitchState		= 0x9C							; state of the switches pressed
															; bit 0	right switch pressed
															; bit 1 left switch pressed
															; bit 2 encoder switch pressed
															; bit 3 encoder turn right state
.equ	ModelAdress		= 0x9D								; adress of the model to be loaded
.equ	MemCnt			= 0x9E								; count the actual byte no sent to the external eeprom
.equ	ModelName		= 0x9F							; 8 subsequent registers to store the model name
.equ	Ch11_1Mult		= 0xA7							; input channel multiplier factors
;.equ	Ch12_1Mult		= 0xAF
;.equ	Ch13_1Mult		= 0xB7
;.equ	Ch14_1Mult		= 0xBF
;.equ	Ch15_1Mult		= 0xC7
;.equ	Ch16_1Mult		= 0xCF
;.equ	Ch17_1Mult		= 0xD7
;.equ	Ch18_1Mult		= 0xDF
.equ	Ch1o_add		= 0xE7
;.equ	Ch2o_add		= 0xE8
;.equ	Ch3o_add		= 0xE9
;.equ	Ch4o_add		= 0xEA
;.equ	Ch5o_add		= 0xEB
;.equ	Ch6o_add		= 0xEC
;.equ	Ch7o_add		= 0xED
.equ	CH8o_add		= 0xEE
.equ	Channel_Min_1	= 0xEF
;.equ	Channel_Min_2	= 0xF0
;.equ	Channel_Min_3	= 0xF1
;.equ	Channel_Min_4	= 0xF2
;.equ	Channel_Min_5	= 0xF3
;.equ	Channel_Min_6	= 0xF4
;.equ	Channel_Min_7	= 0xF5
.equ	Channel_Min_8	= 0xF6
.equ	Channel_Max_1	= 0xF7
;.equ	Channel_Max_2	= 0xF8
;.equ	Channel_Max_3	= 0xF9
;.equ	Channel_Max_4	= 0xFA
;.equ	Channel_Max_5	= 0xFB
;.equ	Channel_Max_6	= 0xFC
;.equ	Channel_Max_7	= 0xFD
.equ	Channel_Max_8	= 0xFE
.equ	MasterOverlay	= 0xFF							; channels used by the master transmitter
.equ	Channel1_1HT	= 0x100							; temporary channel1 storage register to remove spikes from the input signal
.equ	Channel1_1LT	= 0x101
.equ	Channel1_2HT	= 0x102
.equ	Channel1_2LT	= 0x103
.equ	Channel1_3HT	= 0x104
.equ	Channel1_3LT	= 0x105
.equ	Channel1_4HT	= 0x106
.equ	Channel1_4LT	= 0x107
.equ	Channel1_5HT	= 0x108
.equ	Channel1_5LT	= 0x109
.equ	Channel1_6HT	= 0x10A
.equ	Channel1_6LT	= 0x10B
.equ	Channel1_7HT	= 0x10C
.equ	Channel1_7LT	= 0x10D
.equ	Channel1_8HT	= 0x10E
.equ	Channel1_8LT	= 0x10F
.equ	Channel2_1LT	= 0x110							; temporary channel2 storage register to remove spikes from the input signal
.equ	Channel2_1HT	= 0x111
.equ	Channel2_2LT	= 0x112
.equ	Channel2_2HT	= 0x113
.equ	Channel2_3LT	= 0x114
.equ	Channel2_3HT	= 0x115
.equ	Channel2_4LT	= 0x116
.equ	Channel2_4HT	= 0x117
.equ	Channel2_5LT	= 0x118
.equ	Channel2_5HT	= 0x119
.equ	Channel2_6LT	= 0x11A
.equ	Channel2_6HT	= 0x11B
.equ	Channel2_7LT	= 0x11C
.equ	Channel2_7HT	= 0x11D
.equ	Channel2_8LT	= 0x11E
.equ	Channel2_8HT	= 0x11F

;*****************************************************************************************
;	Werte
;*****************************************************************************************

.equ	newtralo		= 1500*CPUFREQ/8000				; pulse time difference for newtral output, 1500us
.equ	newtral1		= 1500*CPUFREQ/8000				; newtral position of all input channels stage 1, 1500us
.equ	pulseduration	= 300*CPUFREQ/8000				; output pulse duration, 300us
.equ	MaxWaitPulse	= CPUFREQ/8*5;26/10				; max. time to wait for a pulse before pulse storage finished and output is started, 5ms
.equ	MulNormval		= 0b10100100					; the product 100d*100d is 2710h. shifting 2710h multiplying the resulting high byte by this normalisation value leady to the correct output value 400d.
.equ	OutSequenceTime	= CPUFREQ/8*23					; time of the complete output sequence including all wait times, 23ms
.equ	MemoryRead		= 0b10100001					; adress the external eeprom and send read byte
.equ	MemoryWrite		= 0b10100000					; adress the external eeprom and send write byte
.equ	ModelBytes		= 99							; number of bytes that give a complete data set for one model
.equ	JitterThreshold	= 400							; threshold to detect jitter of the input value

;*****************************************************************************************
;	Programmcode
;*****************************************************************************************
.cseg
.org	$0000

;*****************************************************************************************
;	Interruptroutinen
;*****************************************************************************************

		rjmp	START									; reset vector
		rjmp	INT0ROUTINE								; Int0 Pin
		reti;	INT1ROUTINE								; Int1 Pin
		reti;	TIM2COMPROUTINE							; Timer/Counter2 Compare event
		reti;	TIM2OVFROUTINE							; Timer/Counter2 Overflow
		rjmp	TIM1CAPTROUTINE							; Timer/Counter1 Capture
		rjmp	TIM1COMPAROUTINE						; Timer/Counter1 Compare A
		rjmp	TIM1COMPBROUTINE						; Timer/Counter1 Compare B
		rjmp	TIM1OVFROUTINE							; Timer/Counter1 Overflow
		rjmp	TIM0OVFROUTINE							; Timer/Counter0 Overflow
		reti;	SPIROUTINE								; Serial Transfer Complete
		reti;	RXCROUTINE								; USART0 Rx complete
		reti;	UDREROUTINE								; USART0, UDRE
		reti;	TXCROUTINE								; USART0 Tx complete
		reti;	ADCROUTINE								; ADC Complete
		reti;	EEREADYROUTINE							; EEPROM ready
		reti;	ANACOMPROUTINE							; analog comparator event
		rjmp	TWIROUTINE								; 2 wire serial interface interrupt
		reti;	SPMREADYROUTINE							; SPM finished

;*****************************************************************************************
;	Programmstart
;*****************************************************************************************

START:
		ldi		Temp, LOW(RAMEND)						; init stack pointer
		ldi		Temp2, HIGH(RAMEND)
		out		SPH, Temp2
		out		SPL, Temp

		clr		Temp
		mov		Null, Temp								; store a zero
		dec		Temp
		mov		NegOne, Temp							; store a "-1"
		ldi		Temp, 1
		mov		One, Temp								; store a one

		sbi		POUT, PINOUT
		sbi		DOUT, PINOUT							; set transmitter output
		cbi		PRIGHT, PINRIGHT						; set right switch input
		cbi		DRIGHT, PINRIGHT
		cbi		PLEFT, PINLEFT							; set left switch input
		cbi		DLEFT, PINLEFT
		cbi		PCODERSWITCH, PINCODERSWITCH			; set encoder switch input
		cbi		DCODERSWITCH, PINCODERSWITCH
		cbi		PCODER1, PINCODER1						; set encoder inputs
		cbi		DCODER1, PINCODER1
		cbi		PCODER2, PINCODER2
		cbi		DCODER2, PINCODER2
													; init int0 routine
		in		Temp, MCUCR
		sbic	PCODER2, PINCODER2						; test if input high or low
		rjmp	START_HIGH								; input is high
		ori		Temp, 0b00000011						; set interrupt to rising edge
		rjmp	START_AFTER_CODER
START_HIGH:												; set interrupt to falling edge
		ori		Temp, 0b00000010
		andi	Temp, 0b11111110
START_AFTER_CODER:
		out		MCUCR, Temp

		ldi		Temp, 0b01000000						; reset any int0 flag
		out		GIFR, Temp								; INT1,INT0, , , , , ,
		in		Temp, GICR								; enable int0
		ori		Temp, 0b01000000						; INT1,INT0, , , , , ,
		out		GICR, Temp
													; init TMR0
		ldi		Temp, 0b00000101						; set timer 0 prescaler to 1024
		out		TCCR0, Temp								; write the prescaler
													; init TMR1
		ldi		Temp, 0xC0
		out		OCR1BH, Null							; set first T1 compare event
		out		OCR1BL, Temp

		ldi		Temp, 0b00000000
		out		TCCR1A, Temp							; COM1A1, COM1A0, COM1B1, COM1B0, FOC1A, FOC1B, WGM11, WGM10
		ldi		Temp, 0b11000010						; set pos. capture edge, clk prescaler 1/8
		out		TCCR1B, Temp							; ICNC1, ICES1, , WGM13, WGM12, CS12, CS11, CS10
		ldi		Temp, 0b00111100						; enable input capture and output compare interrupts
		out		TIMSK, Temp								; OCIE2, TOIE2, TICIE1, OCIE1A, OCIE1B, TOIE1, , TOIE0
		cbi		ADCSRA, ADEN							; disable the ADC to enable ACMUX
		in		Temp, SFIOR
		ori		Temp, 1<<ACME							; enable the ACMUX
		out		SFIOR, Temp
														; set ADMUX to slave input
		ldi		Temp, MUXI1
		out		ADMUX, Temp								; REFS1, REFS0, ADLAR, , MUX3, MUX2, MUX1, MUX0
		;out		ADMUX, Null
		ldi		Temp, 0b00000100						; set ACIC 1 (for TMR1 capture)
		out		ACSR, Temp								; ACD, ACBG, ACO, ACI, ACIE, ACIC, ACIS1, ACIS0
		in		Temp, TIFR								; reset the IC1F bit
		ori		Temp, 1<<ICF1
		out		TIFR, Temp
		
		rcall	DELAY_20ms
		rcall	DISPLAY_INIT						; init the display
		ldi		Temp, EntryCursRight
		rcall	DISPLAY_SEND_CMD
		ldi		Temp, DispOnCursOff
		rcall	DISPLAY_SEND_CMD

		ldi		Temp, Start1Line
		rcall	DISPLAY_SEND_CMD
		ldi		ZH, HIGH(txtwelcome1*2)
		ldi		ZL, LOW(txtwelcome1*2)
		ldi		Temp2, 16
		rcall	DISPLAY_PRINT_PM
		ldi		Temp, Start2Line
		rcall	DISPLAY_SEND_CMD
		ldi		ZH, HIGH(txtwelcome2*2)
		ldi		ZL, LOW(txtwelcome2*2)
		ldi		Temp2, 16
		rcall	DISPLAY_PRINT_PM
		ldi		Temp, Start3Line
		rcall	DISPLAY_SEND_CMD
		ldi		ZH, HIGH(txtwelcome3*2)
		ldi		ZL, LOW(txtwelcome3*2)
		ldi		Temp2, 16
		rcall	DISPLAY_PRINT_PM
		ldi		Temp, Start4Line
		rcall	DISPLAY_SEND_CMD
		ldi		ZH, HIGH(txtwelcome4*2)
		ldi		ZL, LOW(txtwelcome4*2)
		ldi		Temp2, 16
		rcall	DISPLAY_PRINT_PM

		cli
		rcall	DELAY_1s
		rcall	DELAY_1s
		rcall	DELAY_1s
		rcall	DELAY_1s
		rcall	DELAY_1s
		rcall	DELAY_1s

		sts		Channel1ActPulse, Null
		sts		T1OvfCnt, Null
		sts		ModelAdress, Null

		rcall	CLEAR_NAME

		sts		SwitchState, Null
		sts		Status3, Null							; Status 3 und 4 setzen
		sts		Status4, Null
		clr		Status									; clear the state

		in		Temp, TIFR								; reset the IC1F bit
		ori		Temp, 1<<ICF1
		out		TIFR, Temp

													; init the two-wire interface
		ldi		Temp, 10
		out		TWBR, Temp								; write the bit rate
		ldi		Temp, 0b01000101
		out		TWCR, Temp								; TWINT, TWEA, TWSTA, TWSTO, TWWC, TWEN, , TWIE
		ldi		Temp, 0b00000000						; set prescaler
		out		TWSR, Temp								; TWS7, TWS6, TWS5, TWS4, TWS3, , TWPS1, TWPS0

		ldi		Temp, 0b00000010						; load set model state
		sts		Status3, Temp

		sei												; enable interrupts

MAIN:
		sbrc	Status, 1								; test if display routine should be started
		rcall	DISP_ROUTINE
		sbrc	Status, 0								; test if all pulses read
		rcall	CALC_OUTPUT_SEQUENCE
		rjmp	MAIN

;*****************************************************************************************
;	Interruptroutinen
;***************************************************************************************** 

INT0ROUTINE:											; interrupt pin 0, used by the encoder
		push	Temp
		in		Temp, SREG
		push	Temp
		push	Temp2
		push	Temp3
		push	ZL
		push	ZH
		in		Temp, MCUCR
		sbrc	Temp, 0									; test if rising or falling edge
		rjmp	INT0_RISING								; rising egde occured
														; falling edge occured
		ori		Temp, 0b00000011						; set to rising edge
		out		MCUCR, Temp
		lds		Temp, SwitchState
		andi	Temp, 0b11110111						; prepare switch state
		sbic	PCODER1, PINCODER1						; test if second encoder input high or low
		ori		Temp, 0b00001000						; high, set turn right state
		rjmp	INT0_MENU
 INT0_RISING:											; rising edge occured
		andi	Temp, 0b11111110						; set to falling edge
		out		MCUCR, Temp
		lds		Temp, SwitchState
		andi	Temp, 0b11110111						; prepare switch state
		sbis	PCODER1, PINCODER1						; test if second encoder input high or low
		ori		Temp, 0b00001000						; low, set turn right state
 INT0_MENU:
		sts		SwitchState, Temp						; save the turn right/left state
		lds		Temp, Status3
		sbrs	Temp, 5									; test, if in multiplier submenu
		rjmp	INT0_MAIN_MENU							; no, next test
		lds		Temp, SwitchState						; test for turn right/turn left
		lds		Temp2, Status4							; read the target channel
		mov		Temp3, Temp2
		andi	Temp3, 0b00001111						; remove the source channel
		sbrc	Temp, 3
		rjmp	INT0_MUL_R								; turn right state
		cpi		Temp3, 0								; test, if minimum achieved
		breq	INT0_MU_L_MIN							; yes, set to maximum
		dec		Temp2									; decrease target channel
		sts		Status4, Temp2
		rjmp	INT0_OUT
 INT0_MU_L_MIN:
		ori		Temp2, 0b00000111						; set to maximum value
		sts		Status4, Temp2
		rjmp	INT0_OUT
 INT0_MUL_R:
		cpi		Temp3, 7								; test, if maximum achieved
		breq	INT0_MU_R_MAX							; yes, set to minimum value
		inc		Temp2									; increase the target channel
		sts		Status4, Temp2
		rjmp	INT0_OUT
 INT0_MU_R_MAX:
		andi	Temp2, 0b11110000						; set target channel to minimum
		sts		Status4, Temp2
		rjmp	INT0_OUT
 INT0_MAIN_MENU:
		sbrs	Temp, 4									; test, if in main menu
		rjmp	INT0_M_VALS								; no, next test
		lds		Temp2, SwitchState						; yes, switch between the menus
		mov		Temp3, Temp								; prepare a backup
		andi	Temp3, 0b00001111
		sbrc	Temp2, 3								; test, if turn right state
		rjmp	INT0_MAIN_R								; jump to turn right routine
		cpi		Temp3, 2								; test, if minimum value achieved
		breq	INT0_M_L_MIN
		dec		Temp									; reduce menu value
		sts		Status3, Temp
		rjmp	INT0_OUT
 INT0_M_L_MIN:											; minimum menu achieved, set to maximum
		andi	Temp, 0b11110000
		ori		Temp, 0b00001001						; set to maximum value
		sts		Status3, Temp
		rjmp	INT0_OUT
 INT0_MAIN_R:											; turn right state in main menu
		cpi		Temp3, 9								; test, if maximum value achieved
		breq	INT0_M_R_MAX
		inc		Temp									; increase the value
		sts		Status3, Temp
		rjmp	INT0_OUT
 INT0_M_R_MAX:											; maximum achieved, set to minimum
		andi	Temp, 0b11110000
		ori		Temp, 0b00000010						; set to minimum menu value
		sts		Status3, Temp
		rjmp	INT0_OUT
 INT0_M_VALS:
		ldi		ZH, HIGH(INT0_MENU_JMP)
		ldi		ZL, LOW(INT0_MENU_JMP)
		lds		Temp, Status3
		andi	Temp, 0b00001111
		add		ZL, Temp								; calculate the jump adress
		adc		ZH, Null
		ijmp
 INT0_MENU_JMP:
		rjmp	INT0_STATE0								; state 0, show inputs state
		rjmp	INT0_STATE1								; state 1, show outputs state
		rjmp	INT0_STATE2								; state 2, set model state
		rjmp	INT0_OUT								; state 3, init new model state, do nothing
		rjmp	INT0_STATE4								; state 4, set model name state
		rjmp	INT0_STATE5								; state 5, set min channel value state
		rjmp	INT0_STATE6								; state 6, set max channel value state
		rjmp	INT0_STATE7								; state 7, set channel offset state
		rjmp	INT0_STATE8								; state 8, set channel multipliers state
		rjmp	INT0_STATE9								; state 9, set master channels state
		rjmp	INT0_STATE10							; state 10, show master inputs state
 INT0_STATE0:											; show inputs state
		lds		Temp, SwitchState
		sbrc	Temp, 3									; test if turn left or right
		rjmp	INT0_S0_RIGHT							; turned right
		sts		Status3, One							; turned left, change to state 1 (display outputs)
		rjmp	INT0_OUT
 INT0_S0_RIGHT:
		ldi		Temp, 10
		sts		Status3, Temp							; turned right, change to state 10
		rjmp	INT0_OUT
 INT0_STATE1:											; show outputs state
		lds		Temp, SwitchState
		sbrc	Temp, 3									; test if turn left or right
		rjmp	INT0_S1_RIGHT							; turned right
		ldi		Temp, 10
		sts		Status3, Temp							; turned left, change to state 10
		rjmp	INT0_OUT
 INT0_S1_RIGHT:
		sts		Status3, Null							; turned right, change to state 0 (display inputs)
		rjmp	INT0_OUT
 INT0_STATE2:											; set model state
		lds		Temp2, ModelAdress						; load the model adress
		lds		Temp, SwitchState
		sbrc	Temp, 3									; test if turn left or right
		inc		Temp2
		sbrs	Temp, 3
		dec		Temp2
		sts		ModelAdress, Temp2
		andi	Status, 0b11110011						; set twi state 0 (send adress byte 1)
		in		Temp, TWCR
		ori		Temp, 0b11100101						; send a start condition (read the name)
		out		TWCR, Temp								; TWINT,TWEA,TWSTA,TWSTO,TWWC,TWEN, ,TWIE
		rjmp	INT0_OUT
 INT0_STATE4:											; set model name state
		ldi		ZH, HIGH(ModelName)
		ldi		ZL, LOW(ModelName)
		rjmp	INT0_CHANGE_VALUE						; change and save the value
 INT0_STATE5:											; set min channel value state
		ldi		ZH, HIGH(Channel_Min_1)
		ldi		ZL, LOW(Channel_Min_1)
		rjmp	INT0_CHANGE_VALUE						; change and save the value
 INT0_STATE6:											; set max channel value state
		ldi		ZH, HIGH(Channel_Max_1)
		ldi		ZL, LOW(Channel_Max_1)
		rjmp	INT0_CHANGE_VALUE						; change and save the value
 INT0_STATE7:											; set channel offset state
		ldi		ZH, HIGH(Ch1o_add)
		ldi		ZL, LOW(Ch1o_add)
		rjmp	INT0_CHANGE_VALUE						; change and save the value
 INT0_STATE8:											; set channel multipliers state
		ori		Status, 0b00010000						; a value has been changed, set the state
		ldi		ZH, HIGH(Ch11_1Mult)
		ldi		ZL, LOW(Ch11_1Mult)
		lds		Temp2, Status4							; load upper and lower channel number
		andi	Temp2, 0b00000111						; get the target channel
		lsl		Temp2									; multiply by 8
		lsl		Temp2
		lsl		Temp2
		lds		Temp, Status4
		swap	Temp
		andi	Temp, 0b00001111						; get the source channel
		or	 	Temp, Temp2
		rjmp	INT0_CHANGE_MUL							; change and save the value
 INT0_STATE9:											; set master channels state
		ori		Status, 0b00010000						; a value has been changed, set the state
		lds		Temp, Status4							; load the tagret channel
		ldi		Temp2, 0b10000000						; load a change channel value
		andi	Temp, 0b00000111						; only count from 0 to 7
 INT0_S9_INNER:
		breq	INT0_S9_OUT								; go out if zero achieved
		lsr		Temp2									; move the channel to change
		dec		Temp									; decrease the counter
		rjmp	INT0_S9_INNER							; back to loop
 INT0_S9_OUT:
		lds		Temp, MasterOverlay						; load the master mix info
		eor		Temp, Temp2								; change the value
		sts		MasterOverlay, Temp						; save the new value
		rjmp	INT0_OUT								; ignore the encoder
 INT0_STATE10:
		lds		Temp, SwitchState
		sbrc	Temp, 3									; test if turn left or right
		rjmp	INT0_S10_RIGHT							; turned right
		sts		Status3, Null							; turned left, change to state 0 (display inputs)
		rjmp	INT0_OUT
 INT0_S10_RIGHT:
		sts		Status3, One							; turned left, change to state 1 (display outputs)
		rjmp	INT0_OUT
 INT0_CHANGE_VALUE:										; change and save the modified value
		ori		Status, 0b00010000						; a value has been changed, set state
		lds		Temp, Status4							; load the actual letter/channel
 INT0_CHANGE_MUL:										; to change multipliers, jump here
		add		ZL, Temp
		adc		ZH, Null
		ld		Temp, Z									; load the letter
		lds		Temp2, SwitchState
		sbrc	Temp2, 3								; test if turn right or turn left state
		rjmp	INT0_CH_R								; turn right state
		dec 	Temp									; turn left state
		rjmp	INT0_CH_NEXT
 INT0_CH_R:												; turn right state
		inc		Temp
 INT0_CH_NEXT:
		st		Z, Temp									; save the new letter
 INT0_OUT:
		in		Temp, GICR
		andi	Temp, 0b10111111						; disable the interrupt 0
		out		GICR, Temp
		ldi		Temp, 50
		out		TCNT0, Temp								; setup t0 ovf time
		ldi		Temp, 0b00000001						; clear t0ovf flag
		out		TIFR, Temp
		in		Temp, TIMSK
		ori		Temp, 0b00000001						; enable t0ovf interrupt
		out		TIMSK, Temp
		pop		ZH
		pop		ZL
		pop		Temp3
		pop		Temp2
		pop		Temp
		out		SREG, Temp
		pop		Temp
		reti

INT1ROUTINE:											; interrupt pin 1
		reti

TIM2COMPROUTINE:										; timer2 compare event
		reti

TIM2OVFROUTINE:											; timer2 overflow
		reti

TIM1CAPTROUTINE:										; timer1 capture event, used to measure the input pulse time
		push	Temp
		in		Temp, SREG
		push	Temp
		push	XL
		push	XH
		push	YL
		push	YH
		push	ZL
		push	ZH
		lds		YL, LastCaptureL						; load last capture time
		lds		YH, LastCaptureH
		in		ZL, ICR1L								; load actual capture time
		in		ZH, ICR1H
		push	ZL
		push	ZH
		sub		ZL, YL									; calculate the time difference
		sbc		ZH, YH
		ldi		YL, LOW(newtral1)						; load the input newtral value
		ldi		YH, HIGH(newtral1)
		sub		ZL, YL									; subtract newtral value
		sbc		ZH, YH
		lds		Temp, Channel1ActPulse					; calculate the storageadress
		cpi		Temp, 9									; test if max. number of channels achieved
		brsh	T1CA_DONT_STORE_JMP						; yes, dont store more than 8 channels
		sbrs	Status, 5								; test if wait for sync state
		rjmp	T1CA_DONT_STORE							; wait state, don't store
		cpi		Temp, 0
		breq	T1CA_1ST_EVENT_JMP						; in 1st event only store time and set counter
		sbrc	Status, 6								; test if in channel 1 or in channel 2 state
		rjmp	T1CA_CH2								; channel 2 state
		rjmp	T1CA_CH1
 T1CA_DONT_STORE_JMP:
		rjmp	T1CA_DONT_STORE
 T1CA_1ST_EVENT_JMP:
		rjmp	T1CA_1ST_EVENT
 T1CA_CH1:												; store channel 1 timespan
		ldi		YL, LOW(Channel1_1HT-2)
		ldi		YH, HIGH(Channel1_1HT-2)
		add		YL, Temp								; load temporary register value
		adc		YH, Null
		add		YL, Temp
		adc		YH, Null
		ld		XH, Y+									; load the high value
		ld		XL, Y									; load the low value
		sbiw	YL, 1									; re-init the pointer
		st		Y+, ZH									; store value to temporary register
		st		Y, ZL
		sub		XL, ZL									; calculate the difference of subsequent values
		sbc		XH, ZH
		sbrc	XH, 7									; test if difference is negative
		rjmp	T1CA_CH1_NEG							; yes
		cpi		XH, HIGH(JitterThreshold)				; test if the difference is larger than threshold
		brlo	T1CA_CH1_OK								; no, ok
		brne	T1CA_CH1_SKIP							; yes, skip storage
		cpi		XL, LOW(JitterThreshold)
		brlo	T1CA_CH1_OK								; jitter lower than threshold, ok
		rjmp	T1CA_CH1_SKIP
 T1CA_CH1_NEG:
		cpi		XH, HIGH(-JitterThreshold)
		brlo	T1CA_CH1_SKIP
		brne	T1CA_CH1_OK
		cpi		XL, LOW(-JitterThreshold)
		brlo	T1CA_CH1_SKIP
 T1CA_CH1_OK:
		ldi		YL, LOW(Channel1_1H-2)
		ldi		YH, HIGH(Channel1_1H-2)
		add		YL, Temp								; store the time difference to correct register
		adc		YH, Null
		add		YL, Temp
		adc		YH, Null
		st		Y+, ZH									; store the high value
		st		Y, ZL									; store the low value
		rjmp	T1CA_1ST_EVENT							; go on
 T1CA_CH1_SKIP:
		rjmp	T1CA_1ST_EVENT							; go on without storage
 T1CA_CH2:												; store channel 2 timespan
		ldi		YL, LOW(Channel2_1LT-2)
		ldi		YH, HIGH(Channel2_1LT-2)
		add		YL, Temp								; load temporary register value
		adc		YH, Null
		add		YL, Temp
		adc		YH, Null
		ld		XL, Y+									; load the low value
		ld		XH, Y									; load the high value
		sbiw	YL, 1									; re-init the pointer
		st		Y+, ZL									; store value to temporary register
		st		Y, ZH
		sub		XL, ZL									; calculate the difference of subsequent values
		sbc		XH, ZH
		sbrc	XH, 7									; test if difference is negative
		rjmp	T1CA_CH2_NEG							; yes
		cpi		XH, HIGH(JitterThreshold)				; test if the difference is larger than threshold
		brlo	T1CA_CH2_OK								; no, ok
		brne	T1CA_CH2_SKIP							; yes, skip storage
		cpi		XL, LOW(JitterThreshold)
		brlo	T1CA_CH2_OK								; jitter lower than threshold, ok
		rjmp	T1CA_CH2_SKIP
 T1CA_CH2_NEG:
		cpi		XH, HIGH(-JitterThreshold)
		brlo	T1CA_CH2_SKIP
		brne	T1CA_CH2_OK
		cpi		XL, LOW(-JitterThreshold)
		brlo	T1CA_CH2_SKIP
 T1CA_CH2_OK:
		ldi		YL, LOW(Channel2_1L-2)
		ldi		YH, HIGH(Channel2_1L-2)
		add		YL, Temp								; store the time difference to correct register
		adc		YH, Null
		add		YL, Temp
		adc		YH, Null
		st		Y+, ZL									; store the low value
		st		Y, ZH									; store the high value
		rjmp	T1CA_1ST_EVENT							; go on
 T1CA_CH2_SKIP:
		rjmp	T1CA_1ST_EVENT							; go on without storage
 T1CA_1ST_EVENT:
		inc		Temp
		sts		Channel1ActPulse, Temp					; store the new pulse counter
 T1CA_DONT_STORE:
		pop		ZH
		pop		ZL
		sts		LastCaptureH, ZH						; store actual capture time as last time
		sts		LastCaptureL, ZL
		ldi		YL, LOW(MaxWaitPulse)
		ldi		YH, HIGH(MaxWaitPulse)					; set TIM1COMP for 2.6ms to the next event
		add		ZL, YL									; add the timespan
		adc		ZH, YH
		out		OCR1AH, ZH								; write the new wait value
		out		OCR1AL, ZL
		pop		ZH
		pop		ZL
		pop		YH
		pop		YL
		pop		XH
		pop		XL
		pop		Temp
		out		SREG, Temp
		pop		Temp
		reti

TIM1COMPAROUTINE:										; timer1 compare A event, used to detect the input ready state and to activate the output pulse calculation
														; switch between channel 1 and channel 2 detection
		push	Temp
		in		Temp, SREG
		push	Temp
		ori		Status, 0b00000001						; activate the output sequence calculation
		sts		Channel1ActPulse, Null					; reset the actual pulse counter
		lds		Temp, MasterOverlay
		cpi		Temp, 0									; test if master channels are used
		breq	T1COA_NOMASTER							; no, don't switch between channels
		sbrc	Status, 5								; test if in wait state or not
		rjmp	T1COA_NOWAIT							; no wait state
		ori		Status, 0b00100000
		rjmp	T1COA_OUT
 T1COA_NOWAIT:											; no wait state
		andi	Status, 0b11011111						; change to wait state
		sbrc	Status, 6								; test if in channel 1 state
		rjmp	T1COA_NOW_CH2							; channel 2 state
		ori		Status, 0b01000000						; channel 1 state, change to channel 2
		ldi		Temp, MUXI2
		out		ADMUX, Temp
		rjmp	T1COA_OUT
 T1COA_NOW_CH2:											; channel 2 state
		andi	Status, 0b10111111						; change to channel 1
		ldi		Temp, MUXI1
		out		ADMUX, Temp
		rjmp	T1COA_OUT
 T1COA_NOMASTER:										; don't switch between channels
		andi	Status, 0b10111111						; set channel 1 state
		ori		Status, 0b00100000						; set synchronised state
		ldi		Temp, MUXI1								; set mux to channel 1
		out		ADMUX, Temp
 T1COA_OUT:
		pop		Temp
		out		SREG, Temp
		pop		Temp
		reti

TIM1COMPBROUTINE:										; timer1 compare B event, used to generate the output pulses
		push	Temp
		in		Temp, SREG
		push	Temp
		push	Temp2
		push	ZL
		push	ZH
		sbic	POUT, PINOUT
		rjmp	T1COB_HIGH								; output pin is high
		sbi		POUT, PINOUT							; set output high
		lds		Temp2, ChanneloActPulse					; read actual pulse number
		cpi		Temp2, 8
		brne	T1COB_L_NORMAL							; not last pulse, go on normally
		sts		ChanneloActPulse, Null					; reset actual output pulse number
		lds		Temp, NextSequenceSL
		lds		Temp2, NextSequenceSH
		out		OCR1BH, Temp2							; write next sequence start value if actual sequence finished
		out		OCR1BL, Temp
		rjmp	T1COB_OUT
 T1COB_L_NORMAL:
		ldi		ZL, LOW(Channelo_1L)
		ldi		ZH, HIGH(Channelo_1L)
		add		ZL, Temp2
		adc		ZH, Null
		add		ZL, Temp2
		adc		ZH, Null
		inc		Temp2
		sts		ChanneloActPulse, Temp2					; store new pulse number
		ld		Temp, Z+								; load pulse distance low byte
		ld		Temp2, Z								; load the pulse distance high byte
		in		ZL, OCR1BL								; read actual event time
		in		ZH, OCR1BH
		add		ZL, Temp								; add pulse distance
		adc		ZH, Temp2
		ldi		Temp, LOW(newtralo)
		ldi		Temp2, HIGH(newtralo)
		add		ZL, Temp								; add newtral value
		adc		ZH, Temp2
		ldi		Temp, LOW(PulseDuration)
		ldi		Temp2, HIGH(PulseDuration)
		sub		ZL, Temp								; subtract pulse duration
		sbc		ZH, Temp2
		out		OCR1BH, ZH								; write new event time
		out		OCR1BL, ZL
		rjmp	T1COB_OUT
 T1COB_HIGH:											; output is high
		cbi		POUT, PINOUT							; set output low
		lds		Temp2, ChanneloActPulse					; read actual pulse number
		cpi		Temp2, 0
		brne	T1COB_H_NEXT
		in		ZL, OCR1BL								; first pulse, calculate and store the start of the next sequence
		in		ZH, OCR1BH								; read actual sequence start
		ldi		Temp, LOW(OutSequenceTime)
		ldi		Temp2, HIGH(OutSequenceTime)
		add		ZL, Temp								; add the full sequence time
		adc		ZH, Temp2
		sts		NextSequenceSH, ZH						; store next sequence activation time
		sts		NextSequenceSL, ZL
 T1COB_H_NEXT:											; not first pulse
		in		ZL, OCR1BL
		in		ZH, OCR1BH								; read actual sequence start
		ldi		Temp, LOW(PulseDuration)
		ldi		Temp2, HIGH(PulseDuration)				; add the pulse duration
		add		ZL, Temp
		adc		ZH, Temp2
		out		OCR1BH, ZH								; write new interrupt time
		out		OCR1BL, ZL
 T1COB_OUT:
		pop		ZH
		pop		ZL
		pop		Temp2
		pop		Temp
		out		SREG, Temp
		pop		Temp
		reti

TIM1OVFROUTINE:											; timer1 overflow, used for slow routines like display output
		push	Temp
		in		Temp, SREG
		push	Temp
		lds		Temp, T1OvfCnt
		inc		Temp									; increase the t1ovf counter
		sts		T1OvfCnt, Temp
		andi	Temp, 0b00000011
		brne	T1O_NODISP
		ori		Status, 0b00000010						; activate display state every 4 overflows
 T1O_NODISP:
		pop		Temp
		out		SREG, Temp
		pop		Temp
		reti

TIM0OVFROUTINE:											; timer0 overflow, used for the buttons
		push	Temp
		in		Temp, SREG
		push	Temp
		in		Temp, GICR
		ori		Temp, 0b01000000						; enable int0
		out		GICR, Temp
		ldi		Temp, 0b01000000						; disable int0 flags
		out		GIFR, Temp
		in		Temp, TIMSK
		andi	Temp, 0b11111110						; disable tmr0 interrupt
		out		TIMSK, Temp
		pop		Temp
		out		SREG, Temp
		pop		Temp
		reti

SPIROUTINE:												; SPI TX/RX complete
		reti

RXCROUTINE:												; UART receive complete
		reti

UDREROUTINE:											; UART data register empty
		reti
		
TXCROUTINE:												; UART transmission complete
		reti

ADCROUTINE:												; A/D conversion complete
		reti

EEREADYROUTINE:											; EEPROM ready
		reti

ANACOMPROUTINE:											; analog comparator event
		reti

TWIROUTINE:												; 2 wire serial interface interrupt, used for ext. EEPROM control
		push	Temp
		in		Temp, SREG
		push	Temp
		push	Temp2
		push	ZL
		push	ZH
		in		Temp, TWSR								; get the TWI status
		lsr		Temp
		lsr		Temp
		lsr		Temp									; only use the real status bits
		ldi		ZH, HIGH(TWI_S_TAB)						; calculate the jump adress
		ldi		ZL, LOW(TWI_S_TAB)
		add		ZL, Temp
		adc		ZH, Null
		ijmp
 TWI_S_TAB:
		rjmp	TWI_OUT									; value 0x00 is not possible
		rjmp	TWI_START_TRANSMITTED					; value 0x08, start condition transmitted
		rjmp	TWI_REP_START_TRANSMITTED				; value 0x10, repeated start condition transmitted
		rjmp	TWI_ADDR_ACK							; value 0x18, adress transmitted and acknowledged, write mode
		rjmp	TWI_ADDR_NO_ACK							; value 0x20, adress transmitted, not acknowledged, write mode
		rjmp	TWI_DATA_TRANS_ACK						; value 0x28, data transmitted and acknowledged
		rjmp	TWI_DATA_TRANS_NO_ACK					; value 0x30, data transmitted, not acknowledged
		rjmp	TWI_ARB_LOST							; value 0x38, arbitration lost
		rjmp	TWI_ADDR_R_ACK							; value	0x40, adress transmitted and acknowledged, read mode
		rjmp	TWI_ADDR_R_NO_ACK						; value 0x48, adress transmitted, not acknowledged, read mode
		rjmp	TWI_DATA_READ_ACK						; value 0x50, data received and acknowledge sent
		rjmp	TWI_DATA_READ_NO_ACK					; value 0x58, data received, not acknowledge sent
		rjmp	TWI_ADRESS_RECEIVED						; value 0x60, adress received, acknowledge sent
		rjmp	TWI_ARB_LOST_ADDR_RECEIVED				; value 0x68, arbitration lost, adress received, acknowledge sent
		rjmp	TWI_GENERAL_ADDR_RECEIVED				; value 0x70, general adress call received, acknowledge sent
		rjmp	TWI_ARB_LOST_GEN_ADDR_RECEIVED			; value 0x78, arbitration lost, general adress received, acknowledge sent
		rjmp	TWI_SL_DATA_RECEIVED					; value 0x80, data received in slave mode, acknowledge sent
		rjmp	TWI_SL_DATA_NO_ACK						; value 0x88, data received in slave mode, no acknowledge sent
		rjmp	TWI_SL_GEN_DATA_RECEIVED				; value 0x90, data received being generally adressed, acknowledge sent
		rjmp	TWI_SL_GEN_DATA_NO_ACK					; value 0x98, data received being generally adressed, no acknowledge sent
		rjmp	TWI_SL_STOP_RECEIVED					; value 0xA0, stop or repeated start received while adressed as slave
		rjmp	TWI_SL_ADDR_SEND						; value 0xA8, adressed in slave mode to transmit data
		rjmp	TWI_SL_ARBL_ADDR_SEND					; value 0xB0, arbitration lost, adressed as slave to transmit data, acknowledge sent
		rjmp	TWI_SL_DATA_SENT						; value 0xB8, slave mode, data has been sent, acknowledge received
		rjmp	TWI_SL_DATA_SENT_NO_ACK					; value 0xC0, slave mode, data has been sent, no acknowledge received
		rjmp	TWI_SL_LAST_DATA						; value 0xC8, slave mode, last data byte in TWDR sent, acknowledge received
 TWI_START_TRANSMITTED:								; start condition transmitted, always init for writing data to memory
		ldi		Temp, MemoryWrite						; init adress and write command sending
		out		TWDR, Temp								; write the data
		andi	Status, 0b11110011
		in		Temp, TWCR
		ori		Temp, 0b11000000						; clear the interrupt flag
		andi	Temp, 0b11001111						; clear the start bit
		out		TWCR, Temp
		rjmp	TWI_OUT
 TWI_REP_START_TRANSMITTED:							; repeated start condition transmitted, always init for receiving data from memory
		ldi		Temp, MemoryRead						; init adress and send command receiving
		out		TWDR, Temp								; write the data
		andi	Status, 0b11110011						; first the adress is transmitted from the memory
		sts		MemCnt, Null							; reset the memory counter
		in		Temp, TWCR
		ori		Temp, 0b11000000						; clear the interrupt flag
		andi	Temp, 0b11001111						; clear the start bit
		out		TWCR, Temp
		rjmp	TWI_OUT
 TWI_ADDR_ACK:										; adress transmitted and acknowledged, write mode
		lds		Temp, ModelAdress						; send the high adress byte
		lsr		Temp
		out		TWDR, Temp								; write the data
		in		Temp, TWCR
		ori		Temp, 0b11000000						; clear the interrupt flag
		out		TWCR, Temp
		rjmp	TWI_OUT
 TWI_ADDR_NO_ACK:									; adress transmitted, not acknowledged, write mode
		;in		Temp, TWCR
		;ori		Temp, 0b10000000						; clear the interrupt flag
		;out		TWCR, Temp
		rjmp	TWI_OUT
 TWI_DATA_TRANS_ACK:								; data transmitted and acknowledged
		mov		Temp, Status
		andi	Temp, 0b00001100						; use the transfer state bits
		breq	TWI_DTA_ADRESS							; transmit the low adress, as adress msb is transmitted
		cpi		Temp, 0b00001000
		brlo	TWI_DTA_ADRESS2							; adress lsb transmitted, go on
		rjmp	TWI_DTA_DATA							; transfer data to the eeprom
  TWI_DTA_ADRESS:										; transmit low byte of the adress
		lds		Temp, ModelAdress						; send the adress lsb
		andi	Temp, 0b00000001
		lsr		Temp
		ror		Temp									; make the last bit most significant
		out		TWDR, Temp								; write the data
		ori		Status, 0b00000100						; set adress lsb transfered state
		in		Temp, TWCR
		ori		Temp, 0b11000000						; clear the interrupt flag
		out		TWCR, Temp
		rjmp	TWI_OUT
  TWI_DTA_ADRESS2:										; test if write or read should be initialised
		lds		Temp, Status3							; load the menu state
		andi	Temp, 0b00001111						; use the submenu bits only
		breq	TWI_DTA_A2_WR							; write data if state 0
		andi	Status, 0b11110011						; else change to receive data, send a second start condition
		in		Temp, TWCR
		ori		Temp, 0b11100000						; send a 2nd start condition
		out		TWCR, Temp
		rjmp	TWI_OUT
  TWI_DTA_A2_WR:										; change to write state and write first data byte
		ori		Status, 0b00001000						; change to write data state
		andi	Status, 0b11111011
		sts		MemCnt, Null							; reset the memory counter register
  TWI_DTA_DATA:											; write data to the eeprom
		lds		Temp, MemCnt							; load the memory byte counter
		cpi		Temp, ModelBytes						; test if all data written
		breq	TWI_DTA_DATA_STOP						; yes, send the stop condition
		ldi		ZH, HIGH(ModelName)						; load first model adress
		ldi		ZL, LOW(ModelName)
		add		ZL, Temp
		adc		ZH, Null								; get the correct adress
		ld		Temp2, Z								; load the data
		out		TWDR, Temp2								; write the data
		inc		Temp
		sts		MemCnt, Temp							; increase and store the counter
		in		Temp, TWCR
		ori		Temp, 0b11000000						; clear the interrupt flag
		out		TWCR, Temp
		rjmp	TWI_OUT
  TWI_DTA_DATA_STOP:									; send a stop contition
		in		Temp, TWCR
		ori		Temp, 0b10010000						; set stop bit
		out		TWCR, Temp
		rjmp	TWI_OUT
 TWI_DATA_TRANS_NO_ACK:								; data transmitted, not acknowledged
		;in		Temp, TWCR
		;ori		Temp, 0b10010000						; clear the interrupt flag end send stop condition
		;out		TWCR, Temp
		rjmp	TWI_OUT
 TWI_ARB_LOST:										; arbitration lost
		in		Temp, TWCR
		ori		Temp, 0b10000000						; clear the interrupt flag end send stop condition
		out		TWCR, Temp
		rjmp	TWI_OUT
 TWI_ADDR_R_ACK:									; adress transmitted and acknowledged, read mode
		in		Temp, TWCR
		ori		Temp, 0b10000000						; just reset the interrupt state
		out		TWCR, Temp
		rjmp	TWI_OUT
 TWI_ADDR_R_NO_ACK:									; adress transmitted, not acknowledged, read mode
		;in		Temp, TWCR
		;ori		Temp, 0b10000000						; clear the interrupt flag end send stop condition
		;out		TWCR, Temp
		rjmp	TWI_OUT
 TWI_DATA_READ_ACK:									; data received and acknowledge sent
		in		Temp, TWDR								; read the data
		ldi		ZH, HIGH(ModelName)						; valid data received, store to the correct adress
		ldi		ZL, LOW(ModelName)
		lds		Temp2, MemCnt
		add		ZL, Temp2								; calculate the storage adress
		adc		ZH, Null
		st		Z, Temp									; store the data
		inc		Temp2									; increase the data set counter
		sts		MemCnt, Temp2
		lds		Temp, Status3							; load the menu state
		sbrc	Temp, 4									; test if in main menu
		rjmp	TWI_DRA_ALL								; yes, read all data
		cpi		Temp2, 8								; no, read name only
		breq	TWI_DRA_STOP							; yes, stop
		rjmp	TWI_DRA_NEXT							; no, go on
  TWI_DRA_ALL:
		cpi		Temp2, ModelBytes						; test if all bytes received
		breq	TWI_DRA_STOP							; yes, stop
		rjmp	TWI_DRA_NEXT							; no, go on
  TWI_DRA_NEXT:
		in		Temp, TWCR
		ori		Temp, 0b10000000						; just reset the interrupt state
		out		TWCR, Temp
		rjmp	TWI_OUT
  TWI_DRA_STOP:											; last byte to be transmitted
		in		Temp, TWCR
		ori		Temp, 0b10000000						; reset the interrupt
		andi	Temp, 0b10001111						; disable the aknowledge
		out		TWCR, Temp
		rjmp	TWI_OUT
 TWI_DATA_READ_NO_ACK:								; data received, not acknowledge sent
		in		Temp, TWCR
		ori		Temp, 0b10010000						; clear the interrupt flag end send stop condition
		out		TWCR, Temp
		rjmp	TWI_OUT
 TWI_ADRESS_RECEIVED:								; adress received, acknowledge sent
		rjmp	TWI_OUT
 TWI_ARB_LOST_ADDR_RECEIVED:						; arbitration lost, adress received, acknowledge sent
		rjmp	TWI_OUT
 TWI_GENERAL_ADDR_RECEIVED:							; general adress call received, acknowledge sent
		rjmp	TWI_OUT
 TWI_ARB_LOST_GEN_ADDR_RECEIVED:					; arbitration lost, general adress received, acknowledge sent
		rjmp	TWI_OUT
 TWI_SL_DATA_RECEIVED:								; data received in slave mode, acknowledge sent
		rjmp	TWI_OUT
 TWI_SL_DATA_NO_ACK:								; data received in slave mode, no acknowledge sent
		rjmp	TWI_OUT
 TWI_SL_GEN_DATA_RECEIVED:							; data received being generally adressed, acknowledge sent
		rjmp	TWI_OUT
 TWI_SL_GEN_DATA_NO_ACK:							; data received being generally adressed, no acknowledge sent
		rjmp	TWI_OUT
 TWI_SL_STOP_RECEIVED:								; stop or repeated start received while adressed as slave
		rjmp	TWI_OUT
 TWI_SL_ADDR_SEND:									; adressed in slave mode to transmit data
		rjmp	TWI_OUT
 TWI_SL_ARBL_ADDR_SEND:								; arbitration lost, adressed as slave to transmit data, acknowledge sent
		rjmp	TWI_OUT
 TWI_SL_DATA_SENT:									; slave mode, data has been sent, acknowledge received
		rjmp	TWI_OUT
 TWI_SL_DATA_SENT_NO_ACK:							; slave mode, data has been sent, no acknowledge received
		rjmp	TWI_OUT
 TWI_SL_LAST_DATA:									; slave mode, last data byte in TWDR sent, acknowledge received
		rjmp	TWI_OUT
 TWI_OUT:
		pop		ZH
		pop		ZL
		pop		Temp2
		pop		Temp
		out		SREG, Temp
		pop		Temp
		reti

SPMREADYROUTINE:										; SPM finished
		reti

;*****************************************************************************************
;	Subroutinen
;*****************************************************************************************

CLEAR_NAME:
		ldi		Temp2, ModelBytes						; clear all model registers
		ldi		ZH, HIGH(ModelName)
		ldi		ZL, LOW(ModelName)
		rjmp	S_CL_CH_REGS
CLEAR_MODEL_VALUES:
		ldi		Temp2, (ModelBytes-8)					; clear model registers but not the name
		ldi		ZH, HIGH(Ch11_1Mult)
		ldi		ZL, LOW(Ch11_1Mult)
S_CL_CH_REGS:
		st		Z+, Null
		dec		Temp2
		brne	S_CL_CH_REGS
		ret

CALC_OUTPUT_SEQUENCE:									; calculate the output pulse sequence
		lds		Temp, Status3
		andi	Temp, 0b00011111
		cpi		Temp, 5									; test if in set min value mode
		brne	CAO_TEST_MAX							; no, next test
		ldi		ZH, HIGH(Channel_Min_1)					; adress first minimum value
		ldi		ZL, LOW(Channel_Min_1)
		rjmp	CAO_RW
 CAO_TEST_MAX:
		cpi		Temp, 6									; test if in set max value mode
		brne	CAO										; no, run normally
		ldi		ZH, HIGH(Channel_Max_1)					; adress first maximum value
		ldi		ZL, LOW(Channel_Max_1)
 CAO_RW:												; read and write the values adressed
		ldi		Temp3, 8								; run the loop 8 times
		ldi		YH, HIGH(Channelo_1L)					; adress the first output channel
		ldi		YL, LOW(Channelo_1L)
 CAO_RW_LOOP:
		ld		Temp, Z+								; load the value and post increment
		sbrc	Temp, 7									; test if positive
		ser		Temp2									; no, set high byte negative
		sbrs	Temp, 7									; test if negative
		clr		Temp2									; no, set high byte positive
		lsl		Temp									; multiply by 8
		rol		Temp2
		lsl		Temp
		rol		Temp2
		lsl		Temp
		rol		Temp2
		st		Y+, Temp								; save the low value and post increment
		st		Y+, Temp2								; save the high value and post increment
		dec		Temp3									; decrease the loop counter
		brne	CAO_RW_LOOP								; run the loop until counter is zero
		rjmp	CAO_OUT
 CAO:
		ldi		YH, HIGH(Ch11_1Mult)
		ldi		YL, LOW(Ch11_1Mult)
		ldi		XH, HIGH(Channelo_1L)					; set pointer to first output channel low byte
		ldi		XL, LOW(Channelo_1L)
		ldi		Temp, 8									; 8 outer loops
 COS_LOOP1:
		ldi		ZH, HIGH(Channel1_1H)					; point to first input channel
		ldi		ZL, LOW(Channel1_1H)
		ldi		Temp2, 8								; 8 inner loops
		clr		Temp3
		clr		Temp4									; first clear the result registers
		clr		Temp5
		clr		Temp6
 COS_LOOP2:
		ld		Temp1L, Z+								; load input channel high byte
		ld		Temp2L, Y+								; load channel multiplier
		sbrc	Temp2L, 7								; test if 8bit multiplier positive
		rjmp	MUL1NEG									; 8bit multiplier is negative
		sbrc	Temp1L, 7								; 8bit multiplier positive, test if 16bit multiplier positive
		rjmp	MUL1POS2NEG								; 16bit multiplier is negative
		mul		Temp1L, Temp2L							; both multipliers positive
		add		Temp4, r0
		adc		Temp5, r1
		adc		Temp6, Null
		ld		Temp1L, Z+								; load input channel low byte
		mul		Temp1L, Temp2L
		add		Temp3, r0
		adc		Temp4, r1
		adc		Temp5, Null
		adc		Temp6, Null
		rjmp	MUL_END
 MUL1POS2NEG:											; 8bit multiplier positive, 16bit multiplier negative
		com		Temp1L									; invert high byte
		mul		Temp1L, Temp2L
		sub		Temp4, r0
		sbc		Temp5, r1
		sbc		Temp6, Null
		ld		Temp1L, Z+								; load input channel low byte
		neg		Temp1L									; invert low byte
		mul		Temp1L, Temp2L
		sub		Temp3, r0
		sbc		Temp4, r1
		sbc		Temp5, Null
		sbc		Temp6, Null
		rjmp	MUL_END
 MUL1NEG:												; 8bit multiplier negative
		neg		Temp2L									; invert 8bit multiplier
		sbrc	Temp1L, 7								; test if 16bit multiplier positive
		rjmp	MUL1NEG2NEG								; both multipliers negative
		mul		Temp1L, Temp2L							; 8bit multiplier negative 16bit multiplier positive
		sub		Temp4, r0
		sbc		Temp5, r1
		sbc		Temp6, Null
		ld		Temp1L, Z+								; load input channel low byte
		mul		Temp1L, Temp2L
		sub		Temp3, r0
		sbc		Temp4, r1
		sbc		Temp5, Null
		sbc		Temp6, Null
		rjmp	MUL_END
 MUL1NEG2NEG:											; both multipliers negative
		com		Temp1L
		mul		Temp1L, Temp2L
		add		Temp4, r0
		adc		Temp5, r1
		adc		Temp6, Null
		ld		Temp1L, Z+								; load input channel low byte
		neg		Temp1L									; invert low byte
		mul		Temp1L, Temp2L
		add		Temp3, r0
		adc		Temp4, r1
		adc		Temp5, Null
		adc		Temp6, Null
 MUL_END:
		dec		Temp2									; decrease inner loop counter
		brne	COS_LOOP2								; run inner loop until 8 multiplikations processed
		ldi		Temp7, 0x29								; load normalisation value
		mov		Temp1L, Temp7
		sbrc	Temp6, 7								; test if result is negative
		rjmp	MUL_NORM_NEG							; yes
		mul		Temp1L, Temp6							; no, positive, start with highest byte
		mov		Temp6, r0								; store the high byte result
		mov		Temp7, r1
		mul		Temp1L, Temp5
		mov		Temp5, r0								; add the next result
		add		Temp6, r1
		adc		Temp7, Null
		mul		Temp1L, Temp4
		mov		Temp4, r0								; add the next result
		add		Temp5, r1
		adc		Temp6, Null
		adc		Temp7, Null
		mul		Temp1L, Temp3
		mov		Temp3, r0								; add the next result
		add		Temp4, r1
		adc		Temp5, Null
		adc		Temp6, Null
		adc		Temp7, Null								; now the normalised result is stored in Temp:Temp
		rjmp	COS_ADD_MASTER
 MUL_NORM_NEG:
		clr		Temp7									; clear the additional result register (needed to prevent overflow)
		com		Temp6									; first calculate the absolute value
		com		Temp5
		com		Temp4
		neg		Temp3
		mul		Temp1L, Temp6
		clr		Temp6
		sub		Temp6, r0
		sbc		Temp7, r1
		mul		Temp1L, Temp5
		clr		Temp5
		sub		Temp5, r0
		sbc		Temp6, r1
		sbc		Temp7, Null
		mul		Temp1L, Temp4
		clr		Temp4
		sub		Temp4, r0
		sbc		Temp5, r1
		sbc		Temp6, Null
		sbc		Temp7, Null
		mul		Temp1L, Temp3
		clr		Temp3
		sub		Temp3, r0
		sbc		Temp4, r1
		sbc		Temp5, Null
		sbc		Temp6, Null
		sbc		Temp7, Null
 COS_ADD_MASTER:										; normalise the result and add the master input channel
		mov		Temp3, Temp4							; move one byte back
		mov		Temp4, Temp5
		mov		Temp5, Temp6
		mov		Temp6, Temp7
		asr		Temp6									; move 4 bit back
		ror		Temp5
		ror		Temp4
		ror		Temp3
		asr		Temp6
		ror		Temp5
		ror		Temp4
		ror		Temp3
		asr		Temp6
		ror		Temp5
		ror		Temp4
		ror		Temp3
		asr		Temp6
		ror		Temp5
		ror		Temp4
		ror		Temp3
		lds		Temp1L, MasterOverlay					; load master mixing info
		clr		Temp7
		cp		Temp1L, Temp7							; test if any master channel is active
		breq	COS_AFTER_MASTER						; no, skip the test
		mov		Temp2L, Temp							; load the output channel counter
 MASTER_TEST:											; test if channel should be used
		lsr		Temp1L									; shift to carry
		dec		Temp2L									; decrease counter
		brne	MASTER_TEST								; run loop until correct channel info is in carry byte
		brcc	COS_AFTER_MASTER						; dont mix if channel is inactive
		ldi		ZL, LOW(Channel2_8L+2)
		ldi		ZH, HIGH(Channel2_8L+2)					; calculate the channel
		sub		ZL, Temp
		sbc		ZH, Null
		sub		ZL, Temp
		sbc		ZH, Null
		ld		Temp1L, Z+								; load low byte
		ld		Temp2L, Z+								; load high byte
		sbrc	Temp2L, 7								; test if master is negative
		rjmp	COS_MASTER_NEG							; yes
		add		Temp3, Temp1L							; add master channel
		adc		Temp4, Temp2L
		adc		Temp5, Null
		adc		Temp6, Null
		rjmp	COS_AFTER_MASTER
 COS_MASTER_NEG:										; master channel value is negative
		com		Temp2L									; make it positive
		neg		Temp1L
		sub		Temp3, Temp1L							; subtract master channel
		sbc		Temp4, Temp2L
		sbc		Temp5, Null
		sbc		Temp6, Null
 COS_AFTER_MASTER:
		ldi		ZL, LOW(CH8o_add+1)						; load the offset
		ldi		ZH, HIGH(CH8o_add+1)
		sub		ZL, Temp
		sbc		ZH, Null
		ld		Temp1L, Z
		mov		Temp2L, Null							; load high byte
		sbrc	Temp1L, 7								; test if negative
		rjmp	COS_OFFSET_NEG							; yes
		lsl		Temp1L									; scale the offset
		rol		Temp2L
		lsl		Temp1L
		rol		Temp2L
		lsl		Temp1L
		rol		Temp2L
		add		Temp3, Temp1L							; add the offset
		adc		Temp4, Temp2L
		adc		Temp5, Null
		adc		Temp6, Null
		rjmp	COS_CLIP_TEST
 COS_OFFSET_NEG:
		neg		Temp1L									; make it positive
		lsl		Temp1L									; scale the offset
		rol		Temp2L
		lsl		Temp1L
		rol		Temp2L
		lsl		Temp1L
		rol		Temp2L
		sub		Temp3, Temp1L							; subtract the offset
		sbc		Temp4, Temp2L
		sbc		Temp5, Null
		sbc		Temp6, Null
 COS_CLIP_TEST:
		ldi		ZL, LOW(Channel_Min_8+1)				; test for negative clipping
		ldi		ZH, HIGH(Channel_Min_8+1)
		sub		ZL, Temp
		sbc		ZH, Null
		ld		Temp1L, Z								; load the minimum value
		mov		Temp2L, Null
		sbrc	Temp1L, 7								; test if clip value is negative
		rjmp	COS_MIN_TEST_NEG						; negative clip value
		lsl		Temp1L									; scale minimum value
		rol		Temp2L
		lsl		Temp1L
		rol		Temp2L
		lsl		Temp1L
		rol		Temp2L
		sbrc	Temp6, 7								; positive minimum, test if value is negative
		rjmp	COS_CLIP_MIN							; yes, clip
		cpi		Temp6, 1
		brsh	COS_MIN_NOCLIP							; value is much larger than minimum, dont clip
		cpi		Temp5, 1
		brsh	COS_MIN_NOCLIP							; value is much larger than minimum, dont clip
		cp		Temp2L, Temp4							; verify high byte
		brlo	COS_MIN_NOCLIP							; dont clip if larger
		brne	COS_CLIP_MIN							; clip if lower
		cp		Temp3, Temp1L							; verify low byte
		brsh	COS_MIN_NOCLIP							; dont clip if larger
		rjmp	COS_CLIP_MIN							; otherwise clip
 COS_MIN_TEST_NEG:										; minimum value is negative
		sbrs	Temp6, 7								; test if value is positive
		rjmp	COS_MIN_NOCLIP							; yes, dont clip
		sub		Temp2L, One								; make high byte negative
		lsl		Temp1L									; scale the minimum value
		rol		Temp2L
		lsl		Temp1L
		rol		Temp2L
		lsl		Temp1L
		rol		Temp2L
		cpi		Temp6, 0xff
		brlo	COS_CLIP_MIN							; clip if value is too low
		cpi		Temp5, 0xff
		brlo	COS_CLIP_MIN							; clip if value is too low
		cp		Temp4, Temp2L							; verify high byte
		brlo	COS_CLIP_MIN							; clip if lower
		brne	COS_MIN_NOCLIP							; dont clip if larger
		cp		Temp3, Temp1L							; verify low byte
		brsh	COS_MIN_NOCLIP							; dont clip if larger
 COS_CLIP_MIN:											; clip to the minimum value
		mov		Temp3, Temp1L							; set the value to the minimum
		mov		Temp4, Temp2L
 COS_MIN_NOCLIP:
		ldi		ZL, LOW(Channel_Max_8+1)				; test for positive clipping
		ldi		ZH, HIGH(Channel_Max_8+1)
		sub		ZL, Temp
		sbc		ZH, Null
		ld		Temp1L, Z								; load the maximum value
		mov		Temp2L, Null
		sbrc	Temp1L, 7								; test if clip value is negative
		rjmp	COS_MAX_TEST_NEG						; negative clip value
		lsl		Temp1L									; scale minimum value
		rol		Temp2L
		lsl		Temp1L
		rol		Temp2L
		lsl		Temp1L
		rol		Temp2L
		sbrc	Temp6, 7								; test if value is negative
		rjmp	COS_MAX_NOCLIP							; yes, dont clip
		cpi		Temp6, 1
		brsh	COS_CLIP_MAX							; clip if value is too large
		cpi		Temp5, 1
		brsh	COS_CLIP_MAX							; clip if value is too large
		cp		Temp4, Temp2L							; test high byte
		brlo	COS_MAX_NOCLIP							; dont clip if lower
		brne	COS_CLIP_MAX							; clip if higher
		cp	 	Temp1L, Temp3							; test low byte if equal
		brsh	COS_MAX_NOCLIP							; dont clip if value is not higher
		rjmp	COS_CLIP_MAX							; else clip
 COS_MAX_TEST_NEG:										; maximum value is negative
		sub		Temp2L, One								; load 0xff into high byte
		lsl		Temp1L									; scale minimum value
		rol		Temp2L
		lsl		Temp1L
		rol		Temp2L
		lsl		Temp1L
		rol		Temp2L
		sbrs	Temp6, 7								; test if value is positive
		rjmp	COS_CLIP_MAX							; yes, clip to maximum
		cp		Temp4, Temp2L							; verify high byte
		brlo	COS_MAX_NOCLIP							; dont clip if lower
		brne	COS_CLIP_MAX							; clip if higher
		cp		Temp3, Temp1L							; veriy low byte if equal
		brlo	COS_MAX_NOCLIP							; dont clip if lower
 COS_CLIP_MAX:											; clip to maximum value
		mov		Temp3, Temp1L							; set the value to the maximum
		mov		Temp4, Temp2L
 COS_MAX_NOCLIP:
		st		X+, Temp3								; store low byte
		st		X+, Temp4								; store high byte
		dec		Temp									; decrease outer loop counter
		brne	COS_LOOP1_JMP							; run outer loop until 8 output channels processed
 CAO_OUT:
		andi	Status, 0b11111110						; reset calculation state
		ret
COS_LOOP1_JMP:
		rjmp	COS_LOOP1

CR_PRESSED_JMP:
		rjmp	CR_PRESSED
CR_OUT_JMP:
		ret
		;rjmp	CR_OUT

CHANGE_ROUTINE:										; change values depending on state and keys pressed
		lds		Temp, SwitchState						; read previous switch state
		andi	Temp, 0b00000111						; only use the switch pressed bits
		brne	CR_PRESSED_JMP							; switch already pressed, wait until all switches released
		lds		Temp, SwitchState						; again load the switch state
		sbis	PRIGHT, PINRIGHT						; test if right switch is pressed
		ori		Temp, 0b00000001						; yes, set the state
		sbis	PLEFT, PINLEFT							; test if left switch is pressed
		ori		Temp, 0b00000010						; yes, set the state
		sbis	PCODERSWITCH, PINCODERSWITCH			; test if encoder switch is pressed
		ori		Temp, 0b00000100						; yes, set the state
		sts		SwitchState, Temp						; save the new switch state
		andi	Temp, 0b00000111						; only test the switches
		breq	CR_OUT_JMP								; go out if no switch pressed
		lds		Temp2, Status3							; load the menu state
		sbrs	Temp2, 5								; test if multipliers submenu activated
		rjmp	CR_MAIN_MENU							; no, go to next test
 CR_MUL_SUBMENU:										; in set channel multipliers submenu
		sbrc	Temp, 0									; test if right key pressed
		rjmp	CR_MULSUB_1								; yes
		sbrc	Temp, 1									; test if left key pressed
		rjmp	CR_MULSUB_2								; yes
 		rjmp	CR_OUT									; encoder or no switch pressed, ignore
  CR_MULSUB_1:											; set switch pressed
		andi	Temp2, 0b11001111						; change to set multipliers menu
		sts		Status3, Temp2
		rjmp	CR_OUT
  CR_MULSUB_2:											; home key pressed
		andi	Temp2, 0b11011111						; change to main menu
		sts		Status3, Temp2
		sts		Status4, Null							; clear source and target
		rjmp	CR_OUT
 CR_MAIN_MENU:											; in main menu
		sbrs	Temp2, 4								; test if main menu activated
		rjmp	CR_MENUS								; no, go to next tests
		sbrc	Temp, 1									; test if left switch pressed
		rjmp	CR_MAIN_2								; yes
		sbrc	Temp, 2									; test if encoder switch pressed
		rjmp	CR_MAIN_3								; yes
		rjmp	CR_OUT									; otherwise go out
  CR_MAIN_2:											; left switch pressed in main menu
		andi	Temp2, 0b11100000						; set to normal state
		sts		Status3, Temp2
		sbrs	Status, 4								; test if model values varied
		rjmp	CR_OUT
														; store all model values to external eeprom
		andi	Status, 0b11110011						; set twi state 0 (send adress byte 1)
		in		Temp, TWCR
		ori		Temp, 0b10100101						; send a start condition (read the name)
		out		TWCR, Temp								; TWINT,TWEA,TWSTA,TWSTO,TWWC,TWEN, ,TWIE
		andi	Status, 0b11101111						; reset values modified state
		rjmp	CR_OUT
  CR_MAIN_3:											; encoder switch pressed in main menu
		mov		Temp, Temp2								; make a backup
		andi	Temp, 0b00001111
		cpi		Temp, 8									; test if multipliers menu is the goal
		brne	CR_MAIN_3_NORMAL						; no, use normal routines
		ori		Temp2, 0b00100000						; yes, set mutlipliers submenu state
		sts		Status3, Temp2
		rjmp	CR_OUT
  CR_MAIN_3_NORMAL:
		andi	Temp2, 0b11101111						; activate the submenu
		sts		Status3, Temp2
		andi	Temp2, 0b00001111
		cpi		Temp2, 2								; test if change model state
		brne	CR_OUT_JMP1								; just go out if not in change model state
		andi	Status, 0b11110011						; set twi state 0 (send adress byte 1)
		in		Temp, TWCR								; read the name
		ori		Temp, 0b10100101						; send a start condition (read the name)
		out		TWCR, Temp								; TWINT,TWEA,TWSTA,TWSTO,TWWC,TWEN, ,TWIE
  CR_OUT_JMP1:
		rjmp	CR_OUT
 CR_MENUS:												; eighter in normal mode or in submenu
		cpi		Temp2, 3								; test if init model values state
		brne	CR_MENUS_NORMAL							; no, jump to normal change values routine
		sbrc	Temp, 0									; test if right key pressed
		rcall	CLEAR_MODEL_VALUES						; yes, clear all model values
		ori		Status, 0b00010000						; set value changed state
		sbrs	Temp, 2									; test if encoder key pressed
		rjmp	CR_OUT									; released, go out
		ori		Temp2, 0b00010000						; pressed, change to main menu
		sts		Status3, Temp2
		rjmp	CR_OUT
 CR_MENUS_NORMAL:
		mov		Temp3, Temp2							; prepare the jump adress
		andi	Temp3, 0b00001111
		cpi		Temp3, 2
		brlo	CR_MEN_NORMAL							; in normal modes
		cpi		Temp3, 10
		breq	CR_MEN_NORMAL							; 3rd normal mode
		cpi		Temp3, 8
		breq	CR_MEN_MUL								; in change multipliers menu
		lds		Temp3, Status4							; in other change value menu
		sbrc	Temp, 1									; test if left key pressed
		rjmp	CR_MEN_CV_2								; yes
		sbrc	Temp, 2									; test if encoder key pressed
		rjmp	CR_MEN_CV_3								; yes
		sbrs	Temp, 0									; test if no key pressed
		rjmp	CR_OUT									; no key pressed
		lds		Temp3, Status4							; right key pressed, increase the target value
		inc		Temp3
		andi	Temp3, 0b00000111
		sts		Status4, Temp3
		rjmp	CR_OUT
  CR_MEN_CV_2:											; key 2 pressed decrease target channel
		lds		Temp3, Status4
		dec		Temp3
		andi	Temp3, 0b00000111
		sts		Status4, Temp3
		rjmp	CR_OUT
  CR_MEN_CV_3:											; encoder switch pressed, go back
		ori		Temp2, 0b00010000						; activate the main menu
		sts		Status3, Temp2
		sts		Status4, Null							; clear the target selection
		andi	Temp2, 0b00001111
		cpi		Temp2, 2								; test if in load model state
		brne	CR_MEN_CV3_OUT							; no, go out normally
		andi	Status, 0b11110011						; yes, set twi state 0 (send adress byte 1)
		in		Temp, TWCR
		ori		Temp, 0b10100101						; send a start condition (read the name)
		out		TWCR, Temp								; TWINT,TWEA,TWSTA,TWSTO,TWWC,TWEN, ,TWIE
  CR_MEN_CV3_OUT:
		rjmp	CR_OUT
  CR_MEN_NORMAL:										; normal states
		sbrc	Temp, 2									; test if encoder switch is pressed
		rjmp	CR_MEN_N_3								; yes
		rjmp	CR_OUT									; ignore the rest
  CR_MEN_N_3:											; encoder switch pressed, activate main menu
		ori		Temp2, 0b00010010
		andi	Temp2, 0b11110010
		sts		Status3, Temp2
		rjmp	CR_OUT
  CR_MEN_MUL:											; change multiplier state
		sbrc	Temp, 0									; test if right key pressed
		rjmp	CR_MEN_MUL_1							; yes
		sbrc	Temp, 1									; test if left key pressed
		rjmp	CR_MEN_MUL_2							; yes
		sbrs	Temp, 2									; test if no key pressed
		rjmp	CR_OUT									; no key pressed
		ori		Temp2, 0b00110000						; encoder switch pressed go back to multipliers submenu
		sts		Status3, Temp2
		rjmp	CR_OUT
  CR_MEN_MUL_1:											; right switch pressed, increase the source channel
		lds		Temp3, Status4							; load targets and sources
		lds		Temp2, Status4
		andi	Temp3, 0b11110000						; only use the source channels
		cpi		Temp3, 0b01110000						; test if maximum value achieved
		brlo	CR_MEN_MUL_NORM							; no
		andi	Temp2, 0b00001111						; yes, set to 0
		sts		Status4, Temp2
		rjmp	CR_OUT
  CR_MEN_MUL_NORM:
		swap	Temp2									; no maximum, increase the source
		inc		Temp2
		swap	Temp2
		sts		Status4, Temp2
		rjmp	CR_OUT
  CR_MEN_MUL_2:											; left switch pressed, decrease the source channel
		lds		Temp3, Status4							; load source and target channels
		lds		Temp2, Status4
		andi	Temp3, 0b11110000
		breq	CR_MEN_MUL_2_MIN						; if already 0, set to 7
		swap	Temp2									; not zero
		dec		Temp2									; decrease the source channel
		swap	Temp2
		sts		Status4, Temp2
		rjmp	CR_OUT
  CR_MEN_MUL_2_MIN:										; source already zero
		ori		Temp2, 0b01110000						; set to maximum value
		sts		Status4, Temp2
		rjmp	CR_OUT
 CR_PRESSED:											; switch already pressed, wait until all switches released
		lds		Temp, SwitchState						; read swithc state again
		sbic	PRIGHT, PINRIGHT
		andi	Temp, 0b11111110						; clear state when switch released
		sbic	PLEFT, PINLEFT
		andi	Temp, 0b11111101
		sbic	PCODERSWITCH, PINCODERSWITCH
		andi	Temp, 0b11111011
		sts		SwitchState, Temp						; save the new switch state
		;rjmp	CR_OUT									; go out as switch was presed before
 CR_OUT:
		ret

DISP_ROUTINE:										; main display routine, run the different menu points here
		rcall	CHANGE_ROUTINE							; first test if some keys pressed, and if so vary values/states
		andi	Status, 0b11111101						; reset display state
		ldi		ZL, LOW(DR_STATE_TAB)
		ldi		ZH, HIGH(DR_STATE_TAB)
		lds		Temp, Status3							; load actual state
		andi	Temp, 0b00001111						; use the lower 4 bits
		add		ZL, Temp								; calculate the jump adress
		adc		ZH, Null
		ijmp
 DR_STATE_TAB:
		rjmp	DISP_INPUTS								; state 0, diaplay inputs
		rjmp	DISP_OUTPUTS							; state 1, display outputs
		rjmp	DISP_SET_MODEL							; state 2, set model
		rjmp	DISP_INIT_NEW_MODEL						; state 3, init new model
		rjmp	DISP_NAME								; state 4, set name
		rjmp	DISP_MIN								; state 5, set min value
		rjmp	DISP_MAX								; state 6, set max value
		rjmp	DISP_ADD								; state 7, channel offset
		rjmp	DISP_MUL								; state 8, multiplier value
		rjmp	DISP_MASTER								; state 9, set master channels
		rjmp	DISP_MASTER_INPUTS						; state 10, display master inputs
 DISP_INPUTS:											; display the inputs
		ldi		ZH, HIGH(Channel1_1H)
		ldi		ZL, LOW(Channel1_1H)					; load value adress
		rcall	DISP_8_VALUES_IN						; display the input values
		ldi		Temp, Start3Line						; move cursor to beginning of 3rd line
		rcall	DISPLAY_SEND_CMD
		ldi		ZH, HIGH(txtinputs*2)					; display the inputs text
		ldi		ZL, LOW(txtinputs*2)
		ldi		Temp2, 8								; print 8 letters
		rcall	DISPLAY_PRINT_PM
		ldi		ZH, HIGH(txtclear16*2)					; clear the rest of the line
		ldi		ZL, LOW(txtclear16*2)
		ldi		Temp2, 8								; print 8 letters
		rcall	DISPLAY_PRINT_PM
		ldi		Temp, Start4Line						; move cursor to start of 4th line
		rcall	DISPLAY_SEND_CMD
		ldi		Temp2, 8								; print 8 letters
		rcall	DISPLAY_PRINT_PM						; clear beginning of 4th line
		rjmp	DISP_MODEL_NAME
 DISP_OUTPUTS:											; display the outputs
		ldi		ZH, HIGH(Channelo_1L)
		ldi		ZL, LOW(Channelo_1L)					; load value adress
		rcall	DISP_8_VALUES_OUT						; display the output values
		ldi		Temp, Start3Line						; move cursor to beginning of 3rd line
		rcall	DISPLAY_SEND_CMD
		ldi		ZH, HIGH(txtoutputs*2)					; display the outputs text
		ldi		ZL, LOW(txtoutputs*2)
		ldi		Temp2, 8								; print 8 letters
		rcall	DISPLAY_PRINT_PM
		ldi		ZH, HIGH(txtclear16*2)					; clear the rest of the line
		ldi		ZL, LOW(txtclear16*2)
		ldi		Temp2, 8								; print 8 letters
		rcall	DISPLAY_PRINT_PM
		ldi		Temp, Start4Line						; move cursor to start of 4th line
		rcall	DISPLAY_SEND_CMD
		ldi		Temp2, 8								; print 8 letters
		rcall	DISPLAY_PRINT_PM						; clear beginning of 4th line
		rjmp	DISP_MODEL_NAME
  DISP_MASTER_INPUTS:									; display the outputs
		ldi		ZH, HIGH(Channel2_1L)
		ldi		ZL, LOW(Channel2_1L)					; load value adress
		rcall	DISP_8_VALUES_OUT						; display the output values
		ldi		Temp, Start3Line						; move cursor to beginning of 3rd line
		rcall	DISPLAY_SEND_CMD
		ldi		ZH, HIGH(txtinputs2*2)					; display the outputs text
		ldi		ZL, LOW(txtinputs2*2)
		ldi		Temp2, 8								; print 8 letters
		rcall	DISPLAY_PRINT_PM
		ldi		ZH, HIGH(txtclear16*2)					; clear the rest of the line
		ldi		ZL, LOW(txtclear16*2)
		ldi		Temp2, 8								; print 8 letters
		rcall	DISPLAY_PRINT_PM
		ldi		Temp, Start4Line						; move cursor to start of 4th line
		rcall	DISPLAY_SEND_CMD
		ldi		Temp2, 8								; print 8 letters
		rcall	DISPLAY_PRINT_PM						; clear beginning of 4th line
		rjmp	DISP_MODEL_NAME
 DISP_SET_MODEL:
		ldi		Temp, Start1Line						; move cursor to beginning of 1st line
		rcall	DISPLAY_SEND_CMD
		lds		Temp, ModelAdress						; load the model number
		rcall	CONV_UBYTE_DECDISP						; display the model number
		ldi		ZH, HIGH(txtclear16*2)					; clear the rest of the line
		ldi		ZL, LOW(txtclear16*2)
		ldi		Temp2, 12								; print 12 letters
		rcall	DISPLAY_PRINT_PM
		ldi		Temp, Start2Line						; move cursor to beginning of 2nd line
		rcall	DISPLAY_SEND_CMD
		ldi		ZH, HIGH(txtclear16*2)					; clear the line
		ldi		ZL, LOW(txtclear16*2)
		ldi		Temp2, 16								; print 16 letters
		rcall	DISPLAY_PRINT_PM
		ldi		Temp, Start3Line						; move cursor to beginning of 3rd line
		rcall	DISPLAY_SEND_CMD
		ldi		ZH, HIGH(txtsetmodel*2)					; print set model text
		ldi		ZL, LOW(txtsetmodel*2)
		ldi		Temp2, 8								; print 8 letters
		rcall	DISPLAY_PRINT_PM
		ldi		ZH, HIGH(txtclear16*2)					; clear the rest of the line
		ldi		ZL, LOW(txtclear16*2)
		ldi		Temp2, 8								; print 8 letters
		rcall	DISPLAY_PRINT_PM
		ldi		Temp, Start4Line						; move cursor to beginning of 4th line
		rcall	DISPLAY_SEND_CMD
		ldi		Temp2, 8								; print 8 letters
		rcall	DISPLAY_PRINT_PM						; clear beginning of 4th line
		rjmp	DISP_MODEL_NAME
 DISP_INIT_NEW_MODEL:
		ldi		Temp, Start1Line						; move cursor to beginning of 1st line
		rcall	DISPLAY_SEND_CMD
		ldi		ZH, HIGH(txtclear16*2)					; clear the line
		ldi		ZL, LOW(txtclear16*2)
		ldi		Temp2, 16								; print 16 letters
		rcall	DISPLAY_PRINT_PM
		ldi		Temp, Start2Line						; move cursor to beginning of 2nd line
		rcall	DISPLAY_SEND_CMD
		ldi		ZH, HIGH(txtclear16*2)					; clear the line
		ldi		ZL, LOW(txtclear16*2)
		ldi		Temp2, 16								; print 16 letters
		rcall	DISPLAY_PRINT_PM
		ldi		Temp, Start3Line						; move cursor to beginning of 3rd line
		rcall	DISPLAY_SEND_CMD
		ldi		ZH, HIGH(txtclrmodel*2)					; print clear model text
		ldi		ZL, LOW(txtclrmodel*2)
		ldi		Temp2, 8								; print 8 letters
		rcall	DISPLAY_PRINT_PM
		ldi		ZH, HIGH(txtclear16*2)					; clear the rest of the line
		ldi		ZL, LOW(txtclear16*2)
		ldi		Temp2, 8								; print 8 letters
		rcall	DISPLAY_PRINT_PM
		ldi		Temp, Start4Line						; move cursor to beginning of 4th line
		rcall	DISPLAY_SEND_CMD
		ldi		Temp2, 8								; print 8 letters
		rcall	DISPLAY_PRINT_PM						; clear beginning of 4th line
		rjmp	DISP_MODEL_NAME
 DISP_NAME:
		ldi		Temp, Start1Line						; move cursor to beginning of 1st line
		rcall	DISPLAY_SEND_CMD
		ldi		ZH, HIGH(txtclear16*2)					; clear the line
		ldi		ZL, LOW(txtclear16*2)
		ldi		Temp2, 16								; print 16 letters
		rcall	DISPLAY_PRINT_PM
		ldi		Temp, Start2Line						; move cursor to beginning of 2nd line
		rcall	DISPLAY_SEND_CMD
		ldi		ZH, HIGH(txtclear16*2)					; clear the line
		ldi		ZL, LOW(txtclear16*2)
		ldi		Temp2, 16								; print 16 letters
		rcall	DISPLAY_PRINT_PM
		ldi		Temp, Start3Line						; move cursor to beginning of 3rd line
		rcall	DISPLAY_SEND_CMD
		ldi		ZH, HIGH(txtsetname*2)					; print clear model text
		ldi		ZL, LOW(txtsetname*2)
		ldi		Temp2, 8								; print 8 letters
		rcall	DISPLAY_PRINT_PM
		ldi		ZH, HIGH(txtclear16*2)					; clear the rest of the line
		ldi		ZL, LOW(txtclear16*2)
		ldi		Temp2, 8								; print 8 letters
		rcall	DISPLAY_PRINT_PM
		ldi		Temp, Start4Line						; move cursor to beginning of 4th line
		rcall	DISPLAY_SEND_CMD
		ldi		Temp2, 8								; print 8 letters
		rcall	DISPLAY_PRINT_PM						; clear beginning of 4th line
		rjmp	DISP_MODEL_NAME
 DISP_MIN:
		ldi		Temp, Start1Line						; move cursor to beginning of 1st line
		rcall	DISPLAY_SEND_CMD
		lds		Temp2, Status4
		ldi		ZH, HIGH(Channel_Min_1)
		ldi		ZL, LOW(Channel_Min_1)
		add		ZL, Temp2								; calculate address
		adc		ZH, Null
		ld		Temp, Z									; load value
		rcall	CONV_SBYTE_DECDISP						; send value to display
		ldi		ZH, HIGH(txtclear16*2)					; clear the rest of the line
		ldi		ZL, LOW(txtclear16*2)
		ldi		Temp2, 12								; print 12 letters
		rcall	DISPLAY_PRINT_PM
		ldi		Temp, Start2Line						; move cursor to beginning of 2nd line
		rcall	DISPLAY_SEND_CMD
		ldi		ZH, HIGH(txtclear16*2)					; clear the line
		ldi		ZL, LOW(txtclear16*2)
		ldi		Temp2, 16								; print 16 letters
		rcall	DISPLAY_PRINT_PM
		ldi		Temp, Start3Line						; move cursor to beginning of 3rd line
		rcall	DISPLAY_SEND_CMD
		ldi		ZH, HIGH(txtsetmin*2)					; print the set minimum text
		ldi		ZL, LOW(txtsetmin*2)
		ldi		Temp2, 8								; print 8 letters
		rcall	DISPLAY_PRINT_PM
		ldi		ZH, HIGH(txtchannel*2)					; print the channel text
		ldi		ZL, LOW(txtchannel*2)
		ldi		Temp2, 7								; print 7 letters
		rcall	DISPLAY_PRINT_PM
		lds		Temp2, Status4							; restore channel number
		ldi		Temp, 0b00110001						; convert number to ascii
		add		Temp, Temp2
		rcall	DISPLAY_SEND_CHAR						; send the number
		ldi		Temp, Start4Line						; move cursor to beginning of 4th line
		rcall	DISPLAY_SEND_CMD
		ldi		ZH, HIGH(txtclear16*2)					; clear the first part of the line
		ldi		ZL, LOW(txtclear16*2)
		ldi		Temp2, 8								; print 8 signs
		rcall	DISPLAY_PRINT_PM
		rjmp	DISP_MODEL_NAME							; display the model name
 DISP_MAX:
		ldi		Temp, Start1Line						; move cursor to beginning of 1st line
		rcall	DISPLAY_SEND_CMD
		lds		Temp2, Status4
		ldi		ZH, HIGH(Channel_Max_1)
		ldi		ZL, LOW(Channel_Max_1)
		add		ZL, Temp2								; calculate address
		adc		ZH, Null
		ld		Temp, Z									; load value
		rcall	CONV_SBYTE_DECDISP						; send value to display
		ldi		ZH, HIGH(txtclear16*2)					; clear the rest of the line
		ldi		ZL, LOW(txtclear16*2)
		ldi		Temp2, 12								; print 12 letters
		rcall	DISPLAY_PRINT_PM
		ldi		Temp, Start2Line						; move cursor to beginning of 2nd line
		rcall	DISPLAY_SEND_CMD
		ldi		ZH, HIGH(txtclear16*2)					; clear the line
		ldi		ZL, LOW(txtclear16*2)
		ldi		Temp2, 16								; print 16 letters
		rcall	DISPLAY_PRINT_PM
		ldi		Temp, Start3Line						; move cursor to beginning of 3rd line
		rcall	DISPLAY_SEND_CMD
		ldi		ZH, HIGH(txtsetmax*2)					; print the set minimum text
		ldi		ZL, LOW(txtsetmax*2)
		ldi		Temp2, 8								; print 8 letters
		rcall	DISPLAY_PRINT_PM
		ldi		ZH, HIGH(txtchannel*2)					; print the channel text
		ldi		ZL, LOW(txtchannel*2)
		ldi		Temp2, 7								; print 7 letters
		rcall	DISPLAY_PRINT_PM
		lds		Temp2, Status4							; restore channel number
		ldi		Temp, 0b00110001						; convert number to ascii
		add		Temp, Temp2
		rcall	DISPLAY_SEND_CHAR						; send the number
		ldi		Temp, Start4Line						; move cursor to beginning of 4th line
		rcall	DISPLAY_SEND_CMD
		ldi		ZH, HIGH(txtclear16*2)					; clear the first part of the line
		ldi		ZL, LOW(txtclear16*2)
		ldi		Temp2, 8								; print 8 signs
		rcall	DISPLAY_PRINT_PM
		rjmp	DISP_MODEL_NAME							; display the model name
 DISP_ADD:
		ldi		Temp, Start1Line						; move cursor to beginning of 1st line
		rcall	DISPLAY_SEND_CMD
		lds		Temp2, Status4
		ldi		ZH, HIGH(Ch1o_add)
		ldi		ZL, LOW(Ch1o_add)
		add		ZL, Temp2								; calculate the register address
		adc		ZH, Null
		ld		Temp, Z									; load the value
		rcall	CONV_SBYTE_DECDISP						; send value to display
		ldi		ZH, HIGH(txtclear16*2)					; clear the rest of the line
		ldi		ZL, LOW(txtclear16*2)
		ldi		Temp2, 12								; print 12 letters
		rcall	DISPLAY_PRINT_PM
		ldi		Temp, Start2Line						; move cursor to beginning of 2nd line
		rcall	DISPLAY_SEND_CMD
		ldi		ZH, HIGH(txtclear16*2)					; clear the 2nd line
		ldi		ZL, LOW(txtclear16*2)
		ldi		Temp2, 16								; print 16 letters
		rcall	DISPLAY_PRINT_PM
		ldi		Temp, Start3Line						; move cursor to beginning of 3rd line
		rcall	DISPLAY_SEND_CMD
		ldi		ZH, HIGH(txtoffset*2)					; print the offset text
		ldi		ZL, LOW(txtoffset*2)
		ldi		Temp2, 8								; print 8 letters
		rcall	DISPLAY_PRINT_PM
		ldi		ZH, HIGH(txtchannel*2)					; print the channel text
		ldi		ZL, LOW(txtchannel*2)
		ldi		Temp2, 7								; print 7 letters
		rcall	DISPLAY_PRINT_PM
		lds		Temp2, Status4							; restore channel number
		ldi		Temp, 0b00110001						; convert number to ascii
		add		Temp, Temp2
		rcall	DISPLAY_SEND_CHAR						; send the number
		ldi		Temp, Start4Line						; move cursor to beginning of 4th line
		rcall	DISPLAY_SEND_CMD
		ldi		ZH, HIGH(txtclear16*2)					; clear the first part of the line
		ldi		ZL, LOW(txtclear16*2)
		ldi		Temp2, 8								; print 8 signs
		rcall	DISPLAY_PRINT_PM
		rjmp	DISP_MODEL_NAME							; display the model name
 DISP_MUL:												; vary the channel multipliers
		lds		Temp, Status4							; get the target channel
		swap	Temp
		andi	Temp, 0b11110000						; use target channel
		lsr		Temp									; multiply by 8
		ldi		ZH, HIGH(Ch11_1Mult)
		ldi		ZL, LOW(Ch11_1Mult)
		add		ZL, Temp								; calculate the first goal channel register
		adc		ZH, Null
		rcall	DISP_8_VALUES							; display the multipliers
		ldi		Temp, Start3Line						; move cursor to beginning of 3rd line
		rcall	DISPLAY_SEND_CMD
		ldi		ZH, HIGH(txtmult*2)						; clear the first part of the line
		ldi		ZL, LOW(txtmult*2)
		ldi		Temp2, 11								; print 11 signs
		rcall	DISPLAY_PRINT_PM
		lds		Temp, Status4							; recall the state
		andi	Temp, 0b00001111						; use target channel
		ldi		Temp2, 0b00110001						; convert number to ascii
		add		Temp, Temp2
		rcall	DISPLAY_SEND_CHAR						; send the target channel
		ldi		Temp, 0b01111111						; print an arrow
		rcall	DISPLAY_SEND_CHAR						; send the target channel
		lds		Temp, Status4							; recall the state difference
		andi	Temp, 0b11110000						; use source channel
		swap	Temp									; swap nibbles
		ldi		Temp2, 0b00110001						; convert number to ascii
		add		Temp, Temp2
		rcall	DISPLAY_SEND_CHAR						; send the source channel
		ldi		Temp, 0b00100000						; clear the rest of the line
		rcall	DISPLAY_SEND_CHAR
		ldi		Temp, 0b00100000
		rcall	DISPLAY_SEND_CHAR
		ldi		Temp, Start4Line						; move cursor to beginning of 4th line
		rcall	DISPLAY_SEND_CMD
		ldi		ZH, HIGH(txtclear16*2)					; clear the first part of the line
		ldi		ZL, LOW(txtclear16*2)
		ldi		Temp2, 8								; print 8 signs
		rcall	DISPLAY_PRINT_PM
		rjmp	DISP_MODEL_NAME							; display the model name
 DISP_MASTER:
		ldi		Temp, Start1Line						; move the cursor to beginning of first line
		rcall	DISPLAY_SEND_CMD
		lds		Temp, MasterOverlay						; load the master control mixing information
		rcall	CONV_BYTE_TO_BIN_DISP					; send as binary value to the display
		ldi		ZH, HIGH(txtclear16*2)
		ldi		ZL, LOW(txtclear16*2)
		ldi		Temp2, 8
		rcall	DISPLAY_PRINT_PM						; clear the rest of the line
		ldi		Temp, Start2Line						; move the cursor to beginning of line 2
		rcall	DISPLAY_SEND_CMD
		ldi		ZH, HIGH(txtclear16*2)
		ldi		ZL, LOW(txtclear16*2)
		ldi		Temp2, 16
		rcall	DISPLAY_PRINT_PM						; clear the line
		ldi		Temp, Start3Line						; move the cursor to beginning of 3rd line
		rcall	DISPLAY_SEND_CMD
		ldi		ZH, HIGH(txtsetmaster*2)				; print the set master text
		ldi		ZL, LOW(txtsetmaster*2)
		ldi		Temp2, 8
		rcall	DISPLAY_PRINT_PM
		ldi		ZH, HIGH(txtchannel*2)					; print the channel text
		ldi		ZL, LOW(txtchannel*2)
		ldi		Temp2, 7								; print 7 letters
		rcall	DISPLAY_PRINT_PM
		lds		Temp2, Status4							; restore channel number
		ldi		Temp, 0b00110001						; convert number to ascii
		add		Temp, Temp2
		rcall	DISPLAY_SEND_CHAR						; send the number
		ldi		Temp, Start4Line						; move cursor to beginning of 4th line
		rcall	DISPLAY_SEND_CMD
		ldi		ZH, HIGH(txtclear16*2)					; clear the first part of the line
		ldi		ZL, LOW(txtclear16*2)
		ldi		Temp2, 8								; print 8 signs
		rcall	DISPLAY_PRINT_PM
		;rjmp	DISP_MODEL_NAME							; display the model name
 DISP_MODEL_NAME:										; display the model name
		lds		Temp, Status3
		sbrs	Temp, 4									; check if main menu
		rjmp	DISP_MODEL_NAME_NAME					; no, only display the name
		ldi		Temp, Start4Line						; yes, display the select text
		rcall	DISPLAY_SEND_CMD
		ldi		ZH, HIGH(txtselect*2)					; clear the first part of the line
		ldi		ZL, LOW(txtselect*2)
		ldi		Temp2, 8								; print 8 signs
		rcall	DISPLAY_PRINT_PM
 DISP_MODEL_NAME_NAME:									; just display the name
		ldi		Temp, Mid4Line							; move cursor to middle of 4th line
		rcall	DISPLAY_SEND_CMD
		ldi		ZH, HIGH(ModelName)						; load model name adress
		ldi		ZL, LOW(ModelName)
		ldi		Temp2, 8								; 8 letters
		rcall	DISPLAY_PRINT_MEM						; print the 8 letters
		lds		Temp, Status3
		cpi		Temp, 4									; test if set name state
		brne	DISPROUT_CURS_OFF_OUT					; no, go out
														; yes, switch cursor on
		ldi		Temp, DispOnCursOn						; set cursor on
		rcall	DISPLAY_SEND_CMD
		ldi		Temp, Mid4Line							; move cursor to middle of 4th line
		rcall	DISPLAY_SEND_CMD
		lds		Temp2, Status4							; calculate the number of cursor shifts
		cpi		Temp2, 0
 DISP_NAME_NEXT:
		breq	DISPROUT_OUT							; go out if position correct
		ldi		Temp, ShCursRight
		rcall	DISPLAY_SEND_CMD						; move the cursor right
		dec		Temp2									; decrease counter
		rjmp	DISP_NAME_NEXT							; return to loop
 DISPROUT_CURS_OFF_OUT:
		ldi		Temp, DispOnCursOff						; set cursor off
		rcall	DISPLAY_SEND_CMD
 DISPROUT_OUT:
		ret

DISP_8_VALUES_IN:									; display the 8 input values, the higher byte is stored in the lower adress
		ldi		Temp, Start1Line						; move cursor to beginning of first line
		rcall	DISPLAY_SEND_CMD
		ldi		Temp3, 8								; 8 value have to be displayed
 D8VI_INNER:
		ld		Temp2, Z+
		ld		Temp, Z+
		asr		Temp2									; scale the value
		ror		Temp
		asr		Temp2
		ror		Temp
		asr		Temp2
		ror		Temp
		cpi		Temp2, 0
		breq	D8VI_DISP								; test if the value has to be clipped
		cpi		Temp2, 0xff
		breq	D8VI_DISP								; no clipping required
		sbrs	Temp2, 7								; test if positive clipping
		ldi		Temp, 0x7f								; yes, load maximum value
		sbrc	Temp2, 7								; test if negative clipping
		ldi		Temp, 0x80								; yes, load minimum value
 D8VI_DISP:
		rcall	CONV_SBYTE_DECDISP						; send it
		dec		Temp3									; decrease counter
		cpi		Temp3, 4								; test if line 1 finished
		breq	D8VI_2LINE								; yes, start 2nd line
		cpi		Temp3, 0								; test if all values sent
		brne	D8VI_INNER
		ret
 D8VI_2LINE:
		ldi		Temp, Start2Line						; move cursor to beginning of 2nd line
		rcall	DISPLAY_SEND_CMD
		rjmp	D8VI_INNER								; go back

DISP_8_VALUES_OUT:									; display the 8 output values
		ldi		Temp, Start1Line						; move cursor to beginning of first line
		rcall	DISPLAY_SEND_CMD
		ldi		Temp3, 8								; 8 value have to be displayed
 D8VO_INNER:
		ld		Temp, Z+
		ld		Temp2, Z+
		asr		Temp2									; scale the value
		ror		Temp
		asr		Temp2
		ror		Temp
		asr		Temp2
		ror		Temp
		cpi		Temp2, 0
		breq	D8VO_DISP								; test if the value has to be clipped
		cpi		Temp2, 0xff
		breq	D8VO_DISP								; no clipping required
		sbrs	Temp2, 7								; test if positive clipping
		ldi		Temp, 0x7f								; yes, load maximum value
		sbrc	Temp2, 7								; test if negative clipping
		ldi		Temp, 0x80								; yes, load minimum value
 D8VO_DISP:
		rcall	CONV_SBYTE_DECDISP						; send it
		dec		Temp3									; decrease counter
		cpi		Temp3, 4								; test if line 1 finished
		breq	D8VO_2LINE								; yes, start 2nd line
		cpi		Temp3, 0								; test if all values sent
		brne	D8VO_INNER
		ret
 D8VO_2LINE:
		ldi		Temp, Start2Line						; move cursor to beginning of 2nd line
		rcall	DISPLAY_SEND_CMD
		rjmp	D8VO_INNER								; go back

DISP_8_VALUES:										; display 8 subsequent values stored in [Z], set Z to first adress before calling the routine
		ldi		Temp, Start1Line						; move cursor to beginning of first line
		rcall	DISPLAY_SEND_CMD
		ldi		Temp2, 8								; 8 value have to be displayed
 D8V_INNER:
		ld		Temp, Z+								; load value
		rcall	CONV_SBYTE_DECDISP						; send it
		dec		Temp2									; decrease counter
		cpi		Temp2, 4								; test if line 1 finished
		breq	D8V_2LINE								; yes, start 2nd line
		cpi		Temp2, 0								; test if all values sent
		brne	D8V_INNER								; no, go on
		ret
 D8V_2LINE:
		ldi		Temp, Start2Line						; move cursor to beginning of 2nd line
		rcall	DISPLAY_SEND_CMD
		rjmp	D8V_INNER								; go back

.include "conversion.asm"
.include "delayroutines.asm"
.include "asciidisp.asm"								; routines to control the ascii display
